@@ -3,6 +3,13 @@
 import warnings
 import os
 
+########################################################################
+# Note: imports must be limited to the maximum here, and under no circumstances
+# import a package that creates a background thread at import time. Failure to
+# comply will prevent lisa._unshare._do_unshare() to work correctly, as it
+# cannot work if the process is multithreaded when it is called.
+########################################################################
+
 
 from lisa.version import __version__
 
@@ -15,6 +22,16 @@ warnings.filterwarnings(
     module=fr'{__name__}\..*',
 )
 
+try:
+    from cryptography.utils import CryptographyDeprecationWarning
+except ImportError:
+    pass
+else:
+    warnings.filterwarnings(
+        action='ignore',
+        category=CryptographyDeprecationWarning,
+    )
+
 # When the deprecated APIs are used from __main__ (script or notebook), always
 # show the warning
 warnings.filterwarnings(
@@ -23,14 +40,4 @@ warnings.filterwarnings(
     module=r'__main__',
 )
 
-# Prevent matplotlib from trying to connect to X11 server, for headless testing.
-# Must be done before importing matplotlib.pyplot or pylab
-try:
-    import matplotlib
-except ImportError:
-    pass
-else:
-    if not os.getenv('DISPLAY'):
-        matplotlib.use('Agg')
-
 # vim :set tabstop=4 shiftwidth=4 textwidth=80 expandtab
