@@ -42,8 +42,8 @@ import lisa
 import lisa.analysis
 from lisa.analysis.base import AnalysisHelpers, TraceAnalysisBase
 from lisa.utils import get_subclasses, import_all_submodules, DEPRECATED_MAP, get_sphinx_name, groupby, get_short_doc, order_as
-from lisa.trace import MissingTraceEventError, TraceEventCheckerBase
-from lisa.conf import SimpleMultiSrcConf, TopLevelKeyDesc, KeyDesc, LevelKeyDesc
+from lisa.trace import TraceEventCheckerBase
+from lisa.conf import KeyDesc, SimpleMultiSrcConf, TopLevelKeyDesc
 from lisa.version import format_version
 import lisa._git
 
@@ -315,7 +315,7 @@ def autodoc_process_analysis_plots(app, what, name, obj, options, lines, plot_co
 
     plot_methods = set(itertools.chain.from_iterable(
         subclass.get_plot_methods()
-        for subclass in get_subclasses(TraceAnalysisBase)
+        for subclass in TraceAnalysisBase.get_analysis_classes().values()
     ))
 
     if obj not in plot_methods:
@@ -354,7 +354,7 @@ def autodoc_process_analysis_methods(app, what, name, obj, options, lines):
     """
     methods = {
         func: subclass
-        for subclass in get_subclasses(TraceAnalysisBase)
+        for subclass in TraceAnalysisBase.get_analysis_classes().values()
         for name, func in inspect.getmembers(subclass, callable)
     }
 
@@ -377,6 +377,9 @@ def get_analysis_list(meth_type):
         for entry in get_deprecated_map().values()
     }
 
+    # Ensure all the submodules have been imported
+    TraceAnalysisBase.get_analysis_classes()
+
     for subclass in get_subclasses(AnalysisHelpers):
         class_path = f"{subclass.__module__}.{subclass.__qualname__}"
         if meth_type == 'plot':
@@ -592,18 +595,37 @@ def get_subclasses_bullets(cls, abbrev=True, style=None, only_leaves=False):
     )
 
 
-def make_changelog(repo):
+def make_changelog(repo, since=None, head_release_name='Next release', fmt='rst'):
     """
     Generate a reStructuredText changelog to be included in the documentation.
 
     .. note:: The git repository cannot be a shallow clone, as the changelog is
         extracted from the git history.
+
+    .. note:: The ``refs/notes/changelog`` notes is concatenated at the end of
+        commit messages, and the resulting text is parsed. This allows fixing
+        up changelog entries if markers were forgotten without rewriting the
+        history.
     """
-    release_refs = lisa._git.find_tags(repo, 'v*') + ['HEAD']
+
+    if fmt == 'rst':
+        escape_fmt = escape_rst
+    else:
+        escape_fmt = lambda x: x
+
+
+    notes_ref = 'refs/notes/changelog'
+    release_refs = (
+        ['HEAD'] + (
+            [since]
+            if since else
+            lisa._git.find_tags(repo, 'v*')
+        )
+    )
 
     def update_release_name(name):
         if name == 'HEAD':
-            return 'Next release'
+            return head_release_name
         else:
             return name
 
@@ -617,8 +639,9 @@ def make_changelog(repo):
             ref=f'{x}..{y}',
             grep=commit_pattern,
             regex=True,
+            notes_ref=notes_ref,
         )
-        for x, y in zip(release_refs, release_refs[1:])
+        for x, y in zip(release_refs[1:], release_refs)
     }
 
     release_msgs = {
@@ -626,7 +649,8 @@ def make_changelog(repo):
             lisa._git.get_commit_message(
                 repo=repo,
                 ref=ref,
-                format='%B',
+                notes_ref=notes_ref,
+                format='%B%N',
             ).strip()
             for ref in refs
         ]
@@ -690,8 +714,8 @@ def make_changelog(repo):
         return f'{title}\n{body}'
 
     def format_msg(msg):
-        subject = msg.splitlines()[0]
-        return f'- {subject.strip()}'
+        subject = escape_fmt(msg.splitlines()[0].strip())
+        return f'- {subject}'
 
     rst = '\n\n'.join(
         format_release(name, sections)
@@ -701,4 +725,20 @@ def make_changelog(repo):
     return rst
 
 
+class PlaceHolderRef:
+    """
+    If you got redirected to here, this means that the reference points to
+    something private and undocumented, or is not expected to be even
+    documentable.
+    """
+
+def escape_rst(s):
+    """
+    Escape the string so that it's considered plain reStructuredText input,
+    without any markup even if it contains some. This avoids having to use a
+    literal block that is displayed differently.
+    """
+    # https://docutils.sourceforge.io/docs/ref/rst/restructuredtext.html#escaping-mechanism
+    return re.sub(r'(?=[^\s])([^\\])', r'\\\1', s)
+
 # vim :set tabstop=4 shiftwidth=4 expandtab textwidth=80
