@@ -26,15 +26,12 @@ import docutils.core
 import contextlib
 import warnings
 import itertools
-import weakref
 import copy
 from operator import itemgetter, attrgetter
-from collections.abc import Iterable
 
 import numpy
 import matplotlib
 import matplotlib.pyplot as plt
-from matplotlib.figure import Figure
 # Avoid ambiguity between function name and usual variable name
 from cycler import cycler as make_cycler
 import holoviews as hv
@@ -45,12 +42,10 @@ import bokeh.models.widgets
 import panel as pn
 import panel.widgets
 
-from ipywidgets import widgets
-from IPython.display import display
 
-from lisa.utils import Loggable, get_subclasses, get_doc_url, get_short_doc, split_paragraphs, update_wrapper_doc, guess_format, is_running_ipython, nullcontext, measure_time, optional_kwargs, deprecate, memoized
-from lisa.trace import MissingTraceEventError, PandasDataDesc
-from lisa.notebook import axis_link_dataframes, axis_cursor_delta, WrappingHBox, make_figure, _hv_link_dataframes, _hv_fig_to_pane
+from lisa.utils import Loggable, deprecate, get_doc_url, get_short_doc, get_subclasses, guess_format, is_running_ipython, measure_time, memoized, update_wrapper_doc, _import_all_submodules
+from lisa.trace import PandasDataDesc
+from lisa.notebook import _hv_fig_to_pane, _hv_link_dataframes, axis_cursor_delta, axis_link_dataframes, make_figure
 from lisa._generic import TypedList
 
 # Ensure hv.extension() is called
@@ -72,10 +67,9 @@ class AnalysisHelpers(Loggable, abc.ABC):
         """
         Name of the analysis class.
         """
-        pass
 
     @classmethod
-    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='3.0')
+    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='4.0')
     def setup_plot(cls, width=16, height=4, ncols=1, nrows=1, interactive=None, link_dataframes=None, cursor_delta=None, **kwargs):
         """
         Common helper for setting up a matplotlib plot
@@ -151,7 +145,7 @@ class AnalysisHelpers(Loggable, abc.ABC):
 
     @classmethod
     @contextlib.contextmanager
-    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='3.0')
+    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='4.0')
     def set_axis_cycler(cls, axis, *cyclers):
         """
         Context manager to set cyclers on an axis (and the default cycler as
@@ -220,7 +214,7 @@ class AnalysisHelpers(Loggable, abc.ABC):
 
     @classmethod
     @contextlib.contextmanager
-    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='3.0')
+    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='4.0')
     def set_axis_rc_params(cls, axis, rc_params):
         """
         Context manager to set ``matplotlib.rcParams`` while plotting, and then
@@ -239,7 +233,7 @@ class AnalysisHelpers(Loggable, abc.ABC):
                 matplotlib.rcParams.update(orig)
 
     @classmethod
-    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='3.0')
+    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='4.0')
     def cycle_colors(cls, axis, nr_cycles=1):
         """
         Cycle the axis color cycle ``nr_cycles`` forward
@@ -267,7 +261,7 @@ class AnalysisHelpers(Loggable, abc.ABC):
         axis.set_prop_cycle(make_cycler(color=colors[nr_cycles:] + colors[:nr_cycles]))
 
     @classmethod
-    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='3.0')
+    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='4.0')
     def get_next_color(cls, axis):
         """
         Get the next color that will be used to draw lines on the axis
@@ -362,7 +356,7 @@ class AnalysisHelpers(Loggable, abc.ABC):
                 backend=backend,
             )
 
-    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='3.0')
+    @deprecate('Made irrelevant by the use of holoviews', deprecated_in='2.0', removed_in='4.0')
     def do_plot(self, plotter, axis=None, **kwargs):
         """
         Simple helper for consistent behavior across methods.
@@ -735,7 +729,7 @@ class AnalysisHelpers(Loggable, abc.ABC):
                     if backend == 'matplotlib':
                         hv_fig = hv_fig.opts(fig_rcparams=rc_params)
                     else:
-                        self.get_logger().warning('rc_params is only used with matplotlib backend')
+                        self.logger.warning('rc_params is only used with matplotlib backend')
 
                 # Markers added by lisa.notebook.plot_signal
                 if backend == 'bokeh':
@@ -1094,7 +1088,7 @@ class TraceAnalysisBase(AnalysisHelpers):
     Base class for Analysis modules.
 
     :param trace: input Trace object
-    :type trace: :class:`trace.Trace`
+    :type trace: lisa.trace.Trace
 
     :Design notes:
 
@@ -1181,6 +1175,11 @@ class TraceAnalysisBase(AnalysisHelpers):
 
     @classmethod
     def get_analysis_classes(cls):
+        # Import all the submodules so that we have full visibility over the
+        # subclasses.
+        import lisa.analysis as ana
+        _import_all_submodules(ana.__name__, ana.__path__)
+
         return {
             subcls.name: subcls
             for subcls in get_subclasses(cls)
