@@ -21,17 +21,13 @@ import itertools
 import functools
 import operator
 
-import matplotlib.gridspec as gridspec
-import matplotlib.pyplot as plt
-from matplotlib.ticker import FuncFormatter
 import pandas as pd
 import numpy as np
 import holoviews as hv
 
 from lisa.analysis.base import TraceAnalysisBase
-from lisa.utils import memoized
 from lisa.trace import requires_events, requires_one_event_of, CPU, MissingTraceEventError
-from lisa.datautils import series_integrate, df_refit_index, series_refit_index, series_deduplicate, df_add_delta, series_mean, df_window
+from lisa.datautils import series_integrate, df_refit_index, series_refit_index, series_deduplicate, df_add_delta, series_mean, df_window, df_merge
 from lisa.notebook import plot_signal, _hv_neutral
 
 
@@ -40,7 +36,7 @@ class FrequencyAnalysis(TraceAnalysisBase):
     Support for plotting Frequency Analysis data
 
     :param trace: input Trace object
-    :type trace: :class:`trace.Trace`
+    :type trace: lisa.trace.Trace
     """
 
     name = 'frequency'
@@ -93,7 +89,7 @@ class FrequencyAnalysis(TraceAnalysisBase):
             devlib_df = rename(devlib_df)
 
         def groupby_cpu(df):
-            return df.groupby('cpu', observed=True, sort=False)
+            return df.groupby('cpu', observed=True, sort=False, group_keys=False)
 
         # Get the initial values for each CPU
         def init_freq(df, from_devlib):
@@ -202,7 +198,7 @@ class FrequencyAnalysis(TraceAnalysisBase):
 
         # Compute TOTAL Time
         cluster_freqs = df_add_delta(cluster_freqs, col="total_time", window=self.trace.window)
-        time_df = cluster_freqs[["total_time", "frequency"]].groupby('frequency', observed=True, sort=False).sum()
+        time_df = cluster_freqs[["total_time", "frequency"]].groupby('frequency', observed=True, sort=False, group_keys=False).sum()
 
         # Compute ACTIVE Time
         cluster_active = self.ana.idle.signal_cluster_active(cpus)
@@ -342,26 +338,35 @@ class FrequencyAnalysis(TraceAnalysisBase):
         return series_mean(freq)
 
     @TraceAnalysisBase.cache
-    @requires_events('clock_set_rate', 'clock_enable', 'clock_disable')
+    @requires_events('clk_set_rate', 'clk_enable', 'clk_disable')
     def df_peripheral_clock_effective_rate(self, clk_name):
-        rate_df = self.trace.df_event('clock_set_rate')
-        enable_df = self.trace.df_event('clock_enable')
-        disable_df = self.trace.df_event('clock_disable')
 
-        freq = rate_df[rate_df.clk_name == clk_name]
-        enables = enable_df[enable_df.clk_name == clk_name]
-        disables = disable_df[disable_df.clk_name == clk_name]
+        # Note: the kernel still defines a "clock_*" variant for each of these,
+        # but it's not actually used anywhere in the code. The new "clk_*"
+        # events are the ones we are interested about.
+        rate_df = self.trace.df_event('clk_set_rate')
+        enable_df = self.trace.df_event('clk_enable').copy()
+        disable_df = self.trace.df_event('clk_disable').copy()
 
-        freq = pd.concat([freq, enables, disables], sort=False).sort_index()
+        # Add 'state' for enable and disable events
+        enable_df['state'] = 1
+        disable_df['state'] = 0
+
+        freq = rate_df[rate_df['name'] == clk_name]
+        enables = enable_df[enable_df['name'] == clk_name]
+        disables = disable_df[disable_df['name'] == clk_name]
+
+        freq = df_merge((freq, enables, disables)).ffill()
         freq['start'] = freq.index
-        freq['len'] = (freq.start - freq.start.shift()).fillna(0).shift(-1)
-        # The last value will be NaN, fix to be appropriate length
-        freq.loc[freq.index[-1], 'len'] = self.trace.end - freq.index[-1]
-
-        freq.ffill(inplace=True)
+        df_add_delta(
+            freq,
+            col='len',
+            src_col='start',
+            window=self.trace.window,
+            inplace=True
+        )
         freq['effective_rate'] = np.where(
-            freq['state'] == 0, 0,
-            np.where(freq['state'] == 1, freq['state'], float('nan'))
+            freq['state'] == 0, 0, freq['rate']
         )
         return freq
 
@@ -385,7 +390,7 @@ class FrequencyAnalysis(TraceAnalysisBase):
         If ``sched_overutilized`` events are available, the plots will also
         show the intervals of time where the system was overutilized.
         """
-        logger = self.get_logger()
+        logger = self.logger
         df = self.df_cpu_frequency(cpu)
 
         if "freqs" in self.trace.plat_info:
@@ -573,4 +578,31 @@ class FrequencyAnalysis(TraceAnalysisBase):
             )
         ).cols(1)
 
+    @TraceAnalysisBase.plot_method
+    @df_peripheral_clock_effective_rate.used_events
+    def plot_peripheral_frequency(self, clk_name: str, average: bool=True):
+        """
+        Plot frequency for the specified peripheral clock frequency
+
+        :param clk_name: The clock name for which to plot frequency
+        :type clk_name: str
+
+        :param average: If ``True``, add a horizontal line which is the
+            frequency average.
+        :type average: bool
+
+        """
+        df = self.df_peripheral_clock_effective_rate(clk_name)
+        freq = df['effective_rate']
+        freq = series_refit_index(freq, window=self.trace.window)
+
+        fig = plot_signal(freq, name=f'Frequency of {clk_name} (Hz)')
+
+        if average:
+            avg = series_mean(freq)
+            if avg > 0:
+                fig *= hv.HLine(avg, group='average').opts(color='red')
+
+        return fig
+
 # vim :set tabstop=4 shiftwidth=4 expandtab textwidth=80
