@@ -19,10 +19,8 @@
 import json
 import os
 from operator import itemgetter, attrgetter, mul
-from statistics import mean
 from functools import reduce
 from itertools import chain
-from copy import copy
 from collections.abc import Mapping
 from enum import IntEnum
 
@@ -31,11 +29,11 @@ from pandas.api.types import is_numeric_dtype
 import numpy as np
 import holoviews as hv
 
-from lisa.utils import groupby, memoized, FrozenDict, unzip_into
+from lisa.utils import FrozenDict, memoized, unzip_into
 from lisa.datautils import df_merge
 from lisa.analysis.base import TraceAnalysisBase, AnalysisHelpers
 from lisa.analysis.load_tracking import LoadTrackingAnalysis
-from lisa.trace import requires_events, requires_one_event_of, MissingTraceEventError
+from lisa.trace import MissingTraceEventError, requires_one_event_of
 from lisa.conf import ConfigKeyError
 from lisa.stats import Stats
 from lisa.pelt import PELT_SCALE
@@ -124,7 +122,7 @@ class FunctionsAnalysis(TraceAnalysisBase):
         try:
             return self.df_resolve_ksym(df, *args, **kwargs)
         except ConfigKeyError:
-            self.get_logger().warning(f'Missing symbol addresses, function names will not be resolved: {e}')
+            self.logger.warning(f'Missing symbol addresses, function names will not be resolved: {e}')
             return df
 
     @requires_one_event_of('funcgraph_entry', 'funcgraph_exit')
@@ -590,7 +588,7 @@ class _CallGraph:
         return cls(
             cpu_nodes = {
                 cpu: build_graph(subdf)
-                for cpu, subdf in df.groupby('__cpu', observed=True)
+                for cpu, subdf in df.groupby('__cpu', observed=True, group_keys=False)
             }
         )
 
