@@ -14,11 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import os
-import re
-import glob
 from collections import namedtuple
-from operator import attrgetter
 
 import pandas as pd
 import holoviews as hv
@@ -27,7 +23,7 @@ import numpy as np
 from lisa.analysis.base import AnalysisHelpers, TraceAnalysisBase
 from lisa.datautils import df_filter_task_ids, df_window, df_split_signals
 from lisa.trace import TaskID, requires_events, requires_one_event_of, may_use_events, MissingTraceEventError
-from lisa.utils import deprecate, memoized, order_as
+from lisa.utils import memoized, order_as
 from lisa.analysis.tasks import TasksAnalysis
 from lisa.wlgen.rta import RTA, RTAConf
 from lisa.notebook import plot_signal
@@ -289,7 +285,7 @@ class RTAEventsAnalysis(TraceAnalysisBase):
 
                 return df
 
-            df = df.groupby(['__pid', '__comm'], observed=True).apply(f)
+            df = df.groupby(['__pid', '__comm'], observed=True, group_keys=False).apply(f)
 
         return df
 
@@ -307,9 +303,10 @@ class RTAEventsAnalysis(TraceAnalysisBase):
                                 ascending=False)
         # ... keep only the newest/oldest event
         grouped = df.groupby(
-                ['__comm', '__pid', 'phase', 'event'],
-                observed=True,
-                sort=False
+            ['__comm', '__pid', 'phase', 'event'],
+            observed=True,
+            sort=False,
+            group_keys=False,
         )
         df = grouped.head(1)
 
@@ -619,7 +616,7 @@ class RTAEventsAnalysis(TraceAnalysisBase):
         elif timestamp > last_phase_end:
             raise KeyError(f'timestamp={timestamp} is after last phase end: {last_phase_end}')
 
-        i = df.index.get_loc(timestamp, method='ffill')
+        i = df.index.get_indexer([timestamp], method='ffill')[0]
         return PhaseWindow(*get_info(df.iloc[i]))
 
     ###########################################################################
