@@ -27,8 +27,7 @@ import bokeh.models
 from lisa.analysis.base import TraceAnalysisBase
 from lisa.utils import memoized, kwargs_forwarded_to, deprecate
 from lisa.datautils import df_filter_task_ids, series_rolling_apply, series_refit_index, df_refit_index, df_deduplicate, df_split_signals, df_add_delta, df_window, df_update_duplicates, df_combine_duplicates
-from lisa.trace import requires_events, may_use_events, TaskID, CPU, MissingTraceEventError
-from lisa.pelt import PELT_SCALE
+from lisa.trace import requires_events, will_use_events_from, may_use_events, TaskID, CPU, MissingTraceEventError
 from lisa._generic import TypedList
 from lisa.notebook import _hv_neutral, plot_signal, _hv_twinx
 
@@ -160,7 +159,7 @@ class TasksAnalysis(TraceAnalysisBase):
     Support for Tasks signals analysis.
 
     :param trace: input Trace object
-    :type trace: :class:`trace.Trace`
+    :type trace: lisa.trace.Trace
     """
 
     name = 'tasks'
@@ -205,7 +204,7 @@ class TasksAnalysis(TraceAnalysisBase):
         """
         df = self.trace.df_event('sched_wakeup')
 
-        wakeups = df.groupby('pid', observed=True, sort=False).count()["comm"]
+        wakeups = df.groupby('pid', observed=True, sort=False, group_keys=False).count()["comm"]
         df = pd.DataFrame(wakeups).rename(columns={"comm": "wakeups"})
         df["comm"] = df.index.map(self._get_task_pid_name)
 
@@ -269,7 +268,8 @@ class TasksAnalysis(TraceAnalysisBase):
         return rt_tasks
 
     @requires_events('sched_switch', 'sched_wakeup')
-    @may_use_events('sched_wakeup_new', 'task_rename')
+    @will_use_events_from('task_rename')
+    @may_use_events('sched_wakeup_new')
     def _df_tasks_states(self, tasks=None, return_one_df=False):
         """
         Compute tasks states for all tasks.
@@ -331,7 +331,7 @@ class TasksAnalysis(TraceAnalysisBase):
         next_sw_df["curr_state"] = TaskState.TASK_ACTIVE
         next_sw_df.rename(columns={'next_pid': 'pid', 'next_comm': 'comm'}, inplace=True)
 
-        all_sw_df = prev_sw_df.append(next_sw_df, sort=False)
+        all_sw_df = pd.concat([prev_sw_df, next_sw_df], sort=False)
 
         if add_rename:
             rename_df = get_df('task_rename').rename(
@@ -340,13 +340,13 @@ class TasksAnalysis(TraceAnalysisBase):
                 },
             )[['pid', 'comm']]
             rename_df['curr_state'] = TaskState.TASK_RENAMED
-            all_sw_df = all_sw_df.append(rename_df, sort=False)
+            all_sw_df = pd.concat([all_sw_df, rename_df], sort=False)
 
         # Integer values are prefered here, otherwise the whole column
         # is converted to float64
         all_sw_df['target_cpu'] = -1
 
-        df = all_sw_df.append(wk_df, sort=False)
+        df = pd.concat([all_sw_df, wk_df], sort=False)
         df.sort_index(inplace=True)
         df.rename(columns={'__cpu': 'cpu'}, inplace=True)
 
@@ -381,7 +381,7 @@ class TasksAnalysis(TraceAnalysisBase):
             # amount possible.
             df = df_update_duplicates(df, col='Time', inplace=True)
 
-            grouped = df.groupby('pid', observed=True, sort=False)
+            grouped = df.groupby('pid', observed=True, sort=False, group_keys=False)
             new_columns = dict(
                 next_state=grouped['curr_state'].shift(-1, fill_value=TaskState.TASK_UNKNOWN),
                 # GroupBy.transform() will run the function on each group, and
@@ -446,7 +446,7 @@ class TasksAnalysis(TraceAnalysisBase):
 
         :returns: a :class:`pandas.DataFrame` with:
 
-          * A ``cpu`` column (the CPU where the task was on)
+          * A ``cpu`` column (the CPU where the event took place)
           * A ``pid`` column (the PID of the task)
           * A ``comm`` column (the name of the task)
           * A ``target_cpu`` column (the CPU where the task has been scheduled).
@@ -479,7 +479,7 @@ class TasksAnalysis(TraceAnalysisBase):
 
         :returns: a :class:`pandas.DataFrame` with:
 
-          * A ``cpu`` column (the CPU where the task was on)
+          * A ``cpu`` column (the CPU where the event took place)
           * A ``target_cpu`` column (the CPU where the task has been scheduled).
             Will be ``-1`` for non-wakeup events
           * A ``curr_state`` column (the current task state, see :class:`~TaskState`)
@@ -598,7 +598,7 @@ class TasksAnalysis(TraceAnalysisBase):
         df = df[df['curr_state'] == TaskState.TASK_ACTIVE]
 
         # For each CPU, sum the time spent on each by each task
-        by_cpu = df.groupby('cpu', observed=True, sort=False)
+        by_cpu = df.groupby('cpu', observed=True, sort=False, group_keys=False)
         residency_df = by_cpu['runtime'].sum().to_frame()
 
         # Add runtime for CPUs that did not appear in the window
@@ -633,6 +633,8 @@ class TasksAnalysis(TraceAnalysisBase):
         def get_task_df(task):
             try:
                 df = self.ana.tasks.df_task_total_residency(task)
+            except MissingTraceEventError:
+                raise
             # Not all tasks may be available, e.g. tasks outside the TraceView
             # window
             except Exception:
@@ -1034,7 +1036,8 @@ class TasksAnalysis(TraceAnalysisBase):
     @df_task_activation.used_events
     def _plot_tasks_activation(self, tasks, show_legend=None, cpu: CPU=None, alpha:
             float=None, overlay: bool=False, duration: bool=False, duty_cycle:
-            bool=False, which_cpu: bool=False, height_duty_cycle: bool=False):
+            bool=False, which_cpu: bool=False, height_duty_cycle: bool=False, best_effort=False):
+        logger = self.logger
 
         def ensure_last_rectangle(df):
             # Make sure we will draw the last rectangle, which could be
@@ -1121,9 +1124,14 @@ class TasksAnalysis(TraceAnalysisBase):
 
             return figs
 
-        def check_df(task, df):
+        def check_df(task, df, empty_is_none):
             if df.empty:
-                raise ValueError(f'Could not find events associated to task {task}')
+                msg = f'Could not find events associated to task {task}'
+                if empty_is_none:
+                    logger.debug(msg)
+                    return None
+                else:
+                    raise ValueError(msg)
             else:
                 return ensure_last_rectangle(df)
 
@@ -1136,20 +1144,34 @@ class TasksAnalysis(TraceAnalysisBase):
                 df['active'] *= df['duty_cycle']
 
             data = make_rect_df(df[df['active'] != 0])
-            name_df = self.trace.df_event('sched_switch')
-            name_df = name_df[name_df['next_pid'] == task.pid]
-            names = name_df['next_comm'].reindex(data.index, method='ffill')
+            if data.empty:
+                return data
+            else:
+                name_df = self.trace.df_event('sched_switch')
+                name_df = name_df[name_df['next_pid'] == task.pid]
+                names = name_df['next_comm'].reindex(data.index, method='ffill')
 
-            # Use a string for PID so that holoviews interprets it as
-            # categorical variable, rather than continuous. This is important
-            # for correct color mapping
-            data['pid'] = str(task.pid)
-            data['comm'] = names
-            data['start'] = data.index
-            data['cpu'] = df['cpu']
-            data['duration'] = df['duration']
-            data['duty_cycle'] = df['duty_cycle']
-            return data
+                # If there was no sched_switch with next_pid matching task.pid, we
+                # simply take the last known name of the task, which could
+                # originate from another field or another event.
+                #
+                # Note: This prevent an <NA> value, which makes bokeh choke.
+                last_comm = self.trace.get_task_pid_names(task.pid)[-1]
+
+                if last_comm not in names.cat.categories:
+                    names = names.cat.add_categories([last_comm])
+                names = names.fillna(last_comm)
+
+                # Use a string for PID so that holoviews interprets it as
+                # categorical variable, rather than continuous. This is important
+                # for correct color mapping
+                data['pid'] = str(task.pid)
+                data['comm'] = names
+                data['start'] = data.index
+                data['cpu'] = df['cpu']
+                data['duration'] = df['duration']
+                data['duty_cycle'] = df['duty_cycle']
+                return data
 
         def plot_rect(data):
             if show_legend:
@@ -1205,10 +1227,18 @@ class TasksAnalysis(TraceAnalysisBase):
         task_dfs = {
             task: check_df(
                 task,
-                self.df_task_activation(task, cpu=cpu)
+                self.df_task_activation(task, cpu=cpu),
+                empty_is_none=best_effort,
             )
             for task in tasks
         }
+        if best_effort:
+            task_dfs = {
+                task: df
+                for task, df in task_dfs.items()
+                if df is not None
+            }
+        tasks = sorted(task_dfs.keys())
 
         if show_legend:
             fig = hv.Overlay(
@@ -1284,7 +1314,7 @@ class TasksAnalysis(TraceAnalysisBase):
 
     @TraceAnalysisBase.plot_method
     @_plot_tasks_activation.used_events
-    @kwargs_forwarded_to(_plot_tasks_activation, ignore=['tasks'])
+    @kwargs_forwarded_to(_plot_tasks_activation, ignore=['tasks', 'best_effort'])
     def plot_tasks_activation(self, tasks: TypedList[TaskID]=None, hide_tasks: TypedList[TaskID]=None, which_cpu: bool=True, overlay: bool=False, **kwargs):
         """
         Plot all tasks activations, in a style similar to kernelshark.
@@ -1327,10 +1357,12 @@ class TasksAnalysis(TraceAnalysisBase):
             for task in (hide_tasks or [])
         ))
         if tasks:
+            best_effort = False
             task_ids = list(itertools.chain.from_iterable(
                 map(trace.get_task_ids, tasks)
             ))
         else:
+            best_effort = True
             task_ids = trace.task_ids
 
         full_task_ids = sorted(
@@ -1382,6 +1414,7 @@ class TasksAnalysis(TraceAnalysisBase):
             tasks=task_ids,
             which_cpu=which_cpu,
             overlay=overlay,
+            best_effort=best_effort,
             **kwargs
         ).options(
             title=title
@@ -1390,7 +1423,7 @@ class TasksAnalysis(TraceAnalysisBase):
     @TraceAnalysisBase.plot_method
     @plot_tasks_activation.used_events
     @kwargs_forwarded_to(plot_tasks_activation, ignore=['tasks'])
-    @deprecate('Deprecated since it does not provide anything more than plot_tasks_activation', deprecated_in='2.0', removed_in='3.0', replaced_by=plot_tasks_activation)
+    @deprecate('Deprecated since it does not provide anything more than plot_tasks_activation', deprecated_in='2.0', removed_in='4.0', replaced_by=plot_tasks_activation)
     def plot_task_activation(self, task: TaskID, **kwargs):
         """
         Plot task activations, in a style similar to kernelshark.
