@@ -165,7 +165,6 @@ class KeyDescBase(abc.ABC):
         :param style: When "rst", ResStructuredText formatting may be applied
         :param style: str
         """
-        pass
 
     @abc.abstractmethod
     def validate_val(self, val):
@@ -177,7 +176,6 @@ class KeyDescBase(abc.ABC):
             constraints. Note that constraints should ideally be encoded in the
             type itself, to make help message as straightforward as possible.
         """
-        pass
 
 
 class KeyDesc(KeyDescBase):
@@ -358,7 +356,6 @@ class MissingBaseKeyError(ConfigKeyError):
     """
     Exception raised when a base key needed to compute a derived key is missing.
     """
-    pass
 
 
 class DeferredValueComputationError(ConfigKeyError):
@@ -366,9 +363,9 @@ class DeferredValueComputationError(ConfigKeyError):
     Raised when computing the value of :class:`DeferredValue` lead to an
     exception.
     """
-    def __init__(self, msg, excep, **kwargs):
+    def __init__(self, msg, excep, key=None, src=None):
         self.excep = excep
-        super().__init__(msg, **kwargs)
+        super().__init__(msg, key, src)
 
 
 class KeyComputationRecursionError(ConfigKeyError, RecursionError):
@@ -377,7 +374,6 @@ class KeyComputationRecursionError(ConfigKeyError, RecursionError):
     a given key on a configuration instance, or when a :class:`DeferredValue`
     callback is reentered.
     """
-    pass
 
 
 class DerivedKeyDesc(KeyDesc):
@@ -707,7 +703,6 @@ class NestedTopLevelKeyDesc(TopLevelKeyDescBase):
     """
     Top-level key descriptor, with an arbitrary amount of levels.
     """
-    pass
 
 class MultiSrcConfABC(Serializable, abc.ABC):
     _REGISTERED_TOPLEVEL_KEYS = {}
@@ -1053,7 +1048,6 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
         Class attribute defining the structure of the configuration file, as a
         instance of :class:`TopLevelKeyDescBase`
         """
-        pass
 
     DEFAULT_SRC = {}
     """
@@ -1067,8 +1061,7 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
             src_prio=[],
             parent=None,
         )
-        if conf is not None:
-            self.add_src(src, conf)
+        self.add_src(src, conf)
 
         # Give some preset in the the lowest prio source
         if self.DEFAULT_SRC and add_default_src:
@@ -1239,6 +1232,7 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
                 return self
 
         def format_conf(conf):
+            conf = conf or {}
             # Make sure that mappings won't be too long
             max_mapping_len = 10
             key_val = sorted(conf.items())
@@ -1257,7 +1251,7 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
                 for key, val in key_val
             }
 
-        logger = self.get_logger()
+        logger = self.logger
         if logger.isEnabledFor(logging.DEBUG):
             caller, filename, lineno = get_call_site(1, exclude_caller_module=True)
             logger.debug('{caller} ({filename}:{lineno}) has set source "{src}":\n{conf}'.format(
@@ -1277,6 +1271,7 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
         )
 
     def _add_src(self, src, conf, filter_none=False, fallback=False):
+        conf = conf or {}
         # Filter-out None values, so they won't override actual data from
         # another source
         if filter_none:
@@ -1469,9 +1464,9 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
                 else:
                     excep = DeferredValueComputationError(
                         msg,
-                        key=key_qualname,
-                        excep=e,
-                        src=src,
+                        e,
+                        key_qualname,
+                        src,
                     )
                     # Chain explicitly like "raise X from Y" in case it needs
                     # to be wrapped to be raised later
@@ -1480,7 +1475,7 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
                 if error == 'raise':
                     raise excep from e
                 elif error == 'log':
-                    self.get_logger().error(msg)
+                    self.logger.error(msg)
                     # Setup a bomb that will explode during a later access
                     val = DeferredExcep(excep)
                     validate = False
@@ -1612,7 +1607,7 @@ class MultiSrcConf(MultiSrcConfABC, Loggable, Mapping):
             if eval_deferred:
                 val = self._eval_deferred_val(src, key, error='raise')
 
-        logger = self.get_logger()
+        logger = self.logger
         if not quiet and logger.isEnabledFor(logging.DEBUG):
             caller, filename, lineno = get_call_site(2, exclude_caller_module=True)
             logger.debug('{caller} ({filename}:{lineno}) has used key {key} from source "{src}": {val}'.format(
