@@ -185,13 +185,23 @@ def df_split_signals(df, signal_cols, align_start=False, window=None):
         # it a list
         signal_cols = list(signal_cols)
 
-        for group, signal in df.groupby(signal_cols, observed=True, sort=False):
+        # Avoid this warning:
+        # FutureWarning: In a future version of pandas, a length 1 tuple will
+        # be returned when iterating over a groupby with a grouper equal to a
+        # list of length 1. Don't supply a list with a single grouper to avoid
+        # this warning.
+        if len(signal_cols) == 1:
+            _signal_cols = signal_cols[0]
+        else:
+            _signal_cols = signal_cols
+
+        for group, signal in df.groupby(_signal_cols, observed=True, sort=False, group_keys=False):
             # When only one column is looked at, the group is the value instead of
             # a tuple of values
-            if len(signal_cols) < 2:
-                cols_val = {signal_cols[0]: group}
-            else:
+            if isinstance(group, tuple) :
                 cols_val = dict(zip(signal_cols, group))
+            else:
+                cols_val = {signal_cols[0]: group}
 
             if window:
                 signal = df_refit_index(signal, window=window, method='inclusive')
@@ -216,7 +226,7 @@ def _data_refit_index(data, window, method, clip_window):
     # can push it to the right as much as we want without changing the point at
     # which the transition to that value happened
     if duplicate_last:
-        data = data.append(data.iloc[-1:])
+        data = pd.concat([data, data.iloc[-1:]])
     else:
         # Shallow copy is enough, we only want to replace the index and not the
         # actual data
@@ -316,7 +326,7 @@ def df_squash(df, start, end, column='delta'):
 
     # Tweak the closest previous event to include it in the slice
     if not prev_df.empty and start not in middle_df.index:
-        res_df = res_df.append(prev_df.tail(1))
+        res_df = pd.concat([res_df, prev_df.tail(1)])
         res_df.index = [start]
         e1 = end
 
@@ -326,7 +336,7 @@ def df_squash(df, start, end, column='delta'):
         res_df[column] = min(e1 - start, end - start)
 
     if not middle_df.empty:
-        res_df = res_df.append(middle_df)
+        res_df = pd.concat([res_df, middle_df])
         if end in res_df.index:
             # e_last and s1 collide, ditch e_last
             res_df = res_df.drop([end])
@@ -417,7 +427,7 @@ def df_merge(df_list, drop_columns=None, drop_inplace=False, filter_columns=None
         ]
 
     if any(
-        not (df1.columns & df2.columns).empty
+        not (df1.columns.intersection(df2.columns)).empty
         for (df1, df2)  in itertools.combinations(df_list, 2)
     ):
         df = pd.concat(df_list)
@@ -471,7 +481,7 @@ def df_delta(pre_df, post_df, group_on=None):
     post_df["is_pre"] = False
 
     # Merge on columns common to the two dfs to avoid overlapping of names.
-    on_col = sorted(pre_df.columns & post_df.columns)
+    on_col = sorted(pre_df.columns.intersection(post_df.columns))
 
     # Merging on nullable types converts columns to object.
     # Merging on non-nullable types converts integer/boolean to float.
@@ -495,7 +505,7 @@ def df_delta(pre_df, post_df, group_on=None):
     # In each group, search for a faulty sequence (where pre/post events are
     # not interleaving, e.g. pre1->pre2->post1->post2).
     if group_on:
-        grouped = df.groupby(group_on, observed=True, sort=False)
+        grouped = df.groupby(group_on, observed=True, sort=False, group_keys=False)
     else:
         grouped = df
     if grouped['is_pre'].transform(lambda x: x == x.shift()).any():
@@ -547,7 +557,7 @@ def series_derivate(y, x=None, order=1):
 
     :param x: Series with the `x` data. If ``None``, the index of `y` will be
         used. Note that `y` and `x` are expected to have the same index.
-    :type y: pandas.DataFrame or None
+    :type x: pandas.DataFrame or None
 
     :param order: Order of the derivative (1 is speed, 2 is acceleration etc).
     :type order: int
@@ -573,7 +583,7 @@ def series_integrate(y, x=None, sign=None, method='rect', rect_step='post'):
 
     :param x: Series with the `x` data. If ``None``, the index of `y` will be
         used. Note that `y` and `x` are expected to have the same index.
-    :type y: pandas.DataFrame or None
+    :type x: pandas.DataFrame or None
 
     :param sign: Clip the data for the area in positive
         or negative regions. Can be any of:
@@ -651,12 +661,15 @@ def series_integrate(y, x=None, sign=None, method='rect', rect_step='post'):
         raise ValueError(f'Unsupported "sign": {sign}')
 
     if method == "rect":
-        dx = x.diff()
+        if len(x) <= 1:
+            raise ValueError('Cannot integrate with less than 2 points')
+        else:
+            dx = x.diff()
 
-        if rect_step == "post":
-            dx = dx.shift(-1)
+            if rect_step == "post":
+                dx = dx.shift(-1)
 
-        return (y * dx).sum()
+            return (y * dx).sum()
 
     # Make a DataFrame to make sure all rows stay aligned when we drop NaN,
     # which is needed by all the below methods
@@ -688,7 +701,7 @@ def series_mean(y, x=None, **kwargs):
 
     :param x: Series with the `x` data. If ``None``, the index of `y` will be
         used. Note that `y` and `x` are expected to have the same index.
-    :type y: pandas.DataFrame or None
+    :type x: pandas.DataFrame or None
 
     :Variable keyword arguments: Forwarded to :func:`series_integrate`.
     """
@@ -783,9 +796,9 @@ def _data_window(data, window, method, clip_window):
         method = ('ffill', 'bfill')
 
     elif method == 'exclusive':
-        # Default slicing behaviour of pandas' Float64Index is to be exclusive,
+        # Default slicing behaviour of pandas' float index is to be exclusive,
         # so we can use that knowledge to enable a fast path.
-        if isinstance(data.index, pd.Float64Index):
+        if data.index.dtype.kind == 'f':
             return data[slice(*window)]
 
         method = ('bfill', 'ffill')
@@ -823,8 +836,8 @@ def _get_loc(index, x, method):
     #
     # Also, if the index is not sorted, we need to fall back on the slow path
     # as well. Checking is_monotonic is cheap so it's ok to do it here.
-    if method == 'nearest' or not index.is_monotonic:
-        return index.get_loc(x, method=method)
+    if method == 'nearest' or not index.is_monotonic_increasing:
+        return index.get_indexer([x], method=method)[0]
     else:
         if index.empty:
             raise KeyError(x)
@@ -965,7 +978,7 @@ def df_window_signals(df, window, signals, compress_init=False, clip_window=True
                 start = extra_df.index[-1]
 
             index = list(smallest_increment(start, len(init_df)))
-            index = pd.Float64Index(reversed(index))
+            index = pd.Index(reversed(index), dtype='float64')
             return index
     else:
         def make_init_df_index(init_df):
@@ -1016,7 +1029,7 @@ def series_align_signal(ref, to_align, max_shift=None):
         return pd.Series(series.index).diff().min()
     period = min(get_period(ref), get_period(to_align))
     num = math.ceil((end - start) / period)
-    new_index = pd.Float64Index(np.linspace(start, end, num))
+    new_index = pd.Index(np.linspace(start, end, num), dtype='float64')
 
     to_align = to_align.reindex(new_index, method='ffill')
     ref = ref.reindex(new_index, method='ffill')
@@ -1145,7 +1158,7 @@ def series_envelope_mean(series):
         return (maxs_mean - mins_mean) / 2 + mins_mean
 
 # Keep an alias in place for compatibility
-@deprecate(replaced_by=series_envelope_mean, deprecated_in='2.0', removed_in='3.0')
+@deprecate(replaced_by=series_envelope_mean, deprecated_in='2.0', removed_in='4.0')
 def series_tunnel_mean(*args, **kwargs):
     return series_envelope_mean(*args, **kwargs)
 
@@ -1172,7 +1185,7 @@ def series_rolling_apply(series, func, window, window_float_index=True, center=F
     :type center: bool
 
     :param window_float_index: If ``True``, the series passed to ``func`` will
-        be of type :class:`pandas.Float64Index`, in nanoseconds. Disabling is
+        be of type :class:`pandas.Index` (float64), in nanoseconds. Disabling is
         recommended if the index is not used by ``func`` since it will remove
         the need for a conversion.
     :type window_float_index: bool
@@ -1312,13 +1325,7 @@ def df_update_duplicates(df, col=None, func=None, inplace=False):
     """
 
     def increment(series):
-        array = series.array.copy()
-
-        for i in range(len(array)):
-            arr = array[i:]
-            np.nextafter(arr, math.inf, out=arr)
-
-        return pd.Series(array, index=series.index)
+        return np.nextafter(series, math.inf)
 
     def get_duplicated(series):
         # Keep the first, so we update the second duplicates
@@ -1326,16 +1333,13 @@ def df_update_duplicates(df, col=None, func=None, inplace=False):
         return locs, series.loc[locs]
 
     use_index = col is None
-    # Indices already gets copied with to_series()
-    use_copy = inplace and not use_index
 
-    series = df.index.to_series() if use_index else df[col]
-    series = series.copy() if use_copy else series
+    series = df.index.to_series() if use_index else df[col].copy()
     func = func if func else increment
 
     # Update the values until there is no more duplication
     duplicated_locs, duplicated = get_duplicated(series)
-    while duplicated_locs.any():
+    while not duplicated.empty:
         updated = func(duplicated)
         # Change the values at the points of duplication. Otherwise, take the
         # initial value
@@ -1939,7 +1943,7 @@ def series_convert(series, dtype, nullable=None):
             # which is safe since they are immutable. This reduces the memory
             # used by the final series
             new_cat = basic_decode(cat)
-            x.cat.categories = new_cat
+            x = x.cat.rename_categories(new_cat)
             return astype('string')(x)
 
         pipelines.extend((
@@ -2008,12 +2012,12 @@ def df_find_redundant_cols(df, col, cols=None):
         are used.
     :type cols: str or None
     """
-    grouped = df.groupby(col, observed=True)
+    grouped = df.groupby(col, observed=True, group_keys=False)
     cols = cols or (set(df.columns) - {col})
     return {
         _col: dict(map(
             lambda x: (x[0], x[1][0]),
-            series.iteritems()
+            series.items()
         ))
         for _col, series in (
             (
@@ -2044,6 +2048,10 @@ _SIGNALS = [
     SignalDesc('userspace@cpu_frequency_devlib', ['cpu_id']),
     SignalDesc('sched_compute_energy', ['comm', 'pid']),
 
+    SignalDesc('clk_set_rate', ['name']),
+    SignalDesc('clk_enable', ['name']),
+    SignalDesc('clk_disable', ['name']),
+
     SignalDesc('sched_pelt_se', ['comm', 'pid']),
     SignalDesc('sched_load_se', ['comm', 'pid']),
     SignalDesc('sched_util_est_se', ['comm', 'pid']),
