@@ -68,7 +68,6 @@ def _read_multiple_oneline_files(target, glob_patterns):
 
 class EnergyModelCapacityError(Exception):
     """Used by :meth:`EnergyModel.get_optimal_placements`"""
-    pass
 
 
 class ActiveState(namedtuple('ActiveState', ['capacity', 'power'])):
@@ -178,7 +177,7 @@ class EnergyModelNode(_CpuTree):
     def __init__(self, active_states, idle_states,
                  cpu=None, children=None, name=None):
         super().__init__(cpu, children)
-        logger = self.get_logger()
+        logger = self.logger
 
         def is_monotonic(l, decreasing=False):
             op = operator.ge if decreasing else operator.le
@@ -721,7 +720,7 @@ class EnergyModel(Serializable, Loggable):
 
         num_candidates = len(self.cpus) ** len(tasks)
 
-        logger = self.get_logger()
+        logger = self.logger
         logger.debug(
             f'Searching {num_candidates} configurations for optimal task placement...')
 
@@ -818,7 +817,7 @@ class EnergyModel(Serializable, Loggable):
 
         return em
 
-    @deprecate(replaced_by='lisa.energy_model.LinuxEnergyModel.from_target', deprecated_in='2.0', removed_in='3.0')
+    @deprecate(replaced_by='lisa.energy_model.LinuxEnergyModel.from_target', deprecated_in='2.0', removed_in='4.0')
     @staticmethod
     def from_debugfsEM_target(*args, **kwargs):
         """
@@ -826,7 +825,7 @@ class EnergyModel(Serializable, Loggable):
         """
         return LinuxEnergyModel.from_target(*args, **kwargs)
 
-    @deprecate(replaced_by='lisa.energy_model.LegacyEnergyModel.from_target', deprecated_in='2.0', removed_in='3.0')
+    @deprecate(replaced_by='lisa.energy_model.LegacyEnergyModel.from_target', deprecated_in='2.0', removed_in='4.0')
     @staticmethod
     def from_sd_target(*args, **kwargs):
         """
@@ -869,7 +868,7 @@ class EnergyModel(Serializable, Loggable):
         idle = trace.ana.idle.df_cpus_idle().pivot(columns='cpu')['state']
         freqs = trace.ana.frequency.df_cpus_frequency().pivot(columns='cpu')['frequency']
 
-        inputs = pandas.concat([idle, freqs], axis=1, keys=['idle', 'freq']).ffill()
+        inputs = pandas.concat([idle, freqs], axis=1, keys=['idle', 'freq'], sort=True).ffill()
 
         # Drop stuff at the beginning where we don't have the inputs
         # (e.g. where we have had our first cpu_idle event but no cpu_frequency)
@@ -1283,16 +1282,6 @@ class LegacyEnergyModel(EnergyModel):
             # Again, not strictly necessary, just more pleasant.
             return sorted(ret, key=operator.itemgetter(0))
 
-        # Use cpufreq to figure out the frequency domains
-        freq_domains = []
-        remaining_cpus = cpus.copy()
-        while remaining_cpus:
-            cpu = remaining_cpus.pop()
-            dom = target.cpufreq.get_related_cpus(cpu)
-            freq_domains.append(sorted(dom))
-            remaining_cpus -= set(dom)
-        freq_domains = sorted(freq_domains)
-
         return cls(
             root_node=EnergyModelRoot(
                 children=[
@@ -1326,7 +1315,7 @@ class LegacyEnergyModel(EnergyModel):
                 ],
                 idle_states=[],
             ),
-            freq_domains=freq_domains,
+            freq_domains=sorted(target.cpufreq.iter_domains()),
         )
 
 
