@@ -23,7 +23,6 @@ import functools
 import collections
 import warnings
 import importlib
-import contextlib
 import inspect
 from uuid import uuid4
 from itertools import starmap
@@ -42,7 +41,9 @@ from cycler import cycler as make_cycler
 from ipywidgets import widgets, Layout, interact
 from IPython.display import display
 
-from lisa.utils import is_running_ipython, order_as
+from lisa.utils import is_running_ipython, order_as, destroyablecontextmanager, ContextManagerExit
+
+pn.extension('tabulator')
 
 # Enable all backends, finishing with matplotlib so it becomes the default (for
 # backward compat). Since this also corresponds to holoviews default, we are
@@ -177,7 +178,7 @@ def axis_link_dataframes(axis, df_list, before=1, after=5, cursor_color='red', f
             elif loc > df.index[-1]:
                 iloc = -1
             else:
-                iloc = df.index.get_loc(loc, method='ffill')
+                iloc = df.index.get_indexer([loc], method='ffill')[0]
             index_loc = df.index[iloc]
 
             begin = max(iloc - before, 0)
@@ -375,7 +376,7 @@ def make_figure(width, height, nrows, ncols, interactive=None, **kwargs):
     return (figure, axes)
 
 
-def plot_signal(series, name=None, interpolation=None, add_markers=True):
+def plot_signal(series, name=None, interpolation=None, add_markers=True, vdim=None):
     """
     Plot a signal using ``holoviews`` library.
 
@@ -392,6 +393,9 @@ def plot_signal(series, name=None, interpolation=None, add_markers=True):
 
     :param add_markers: Add markers to the plot.
     :type add_markers: bool
+
+    :param vdim: Value axis dimension.
+    :type vdim: holoviews.Dimension
     """
     if isinstance(series, pd.DataFrame):
         try:
@@ -408,12 +412,16 @@ def plot_signal(series, name=None, interpolation=None, add_markers=True):
         # We don't set the unit as this will prevent shared_axes to work if
         # the other plots do not set the unit, which is what usually
         # happens, since only name/label is taken from pandas index names.
-        hv.Dimension('Time'),
+        hv.Dimension('Time', unit='s'),
     ]
+    vdims = [
+        vdim
+    ] if vdim is not None else None
     fig = hv.Curve(
         series,
         label=label,
         kdims=kdims,
+        vdims=vdims,
     ).opts(
         interpolation=interpolation,
         title=label,
@@ -428,6 +436,7 @@ def plot_signal(series, name=None, interpolation=None, add_markers=True):
             label=label,
             group='marker',
             kdims=kdims,
+            vdims=vdims,
         )
     return fig
 
@@ -538,7 +547,7 @@ def _hv_multi_line_title(fig):
     return fig.options(hooks=[_hv_multi_line_title_hook])
 
 
-@contextlib.contextmanager
+@destroyablecontextmanager
 def _hv_set_backend(backend):
     """
     Context manager to work around this issue:
@@ -551,7 +560,7 @@ def _hv_set_backend(backend):
         # at import time
         hv.Store.set_current_backend(backend)
         yield
-    finally:
+    except ContextManagerExit:
         if old_backend:
             hv.Store.set_current_backend(old_backend)
 
@@ -576,7 +585,7 @@ def _hv_link_dataframes(fig, dfs):
             df.index = df.index.copy(deep=False)
             df.index.name = ''
 
-        df_widget = pn.widgets.DataFrame(
+        df_widget = pn.widgets.Tabulator(
             df,
             name=df.attrs.get('name', f'dataframe #{i}'),
             formatters={
@@ -584,20 +593,19 @@ def _hv_link_dataframes(fig, dfs):
             },
             # Disable edition of the dataframe
             disabled=True,
-            sortable=False,
+            # sortable=False,
             # Ensure some columns are always displayed
-            # Note: Tabulator requires a list of column names instead.
-            frozen_columns=len(event_header) + 1,
+            frozen_columns=event_header,
+            # For pn.widgets.DataFrame.
+            # frozen_columns=len(event_header) + 1,
             height=400,
-            autosize_mode='fit_viewport',
+            # autosize_mode='fit_viewport',
             row_height=25,
+            pagination=None,
 
             # Only relevant for pn.widgets.Tabulator
-            #theme='simple',
-            #selectable='checkbox',
-            # Avoid transferring too much data at once to the browser
-            #pagination='remote',
-            #page_size=100,
+            theme='simple',
+            selectable='toggle',
         )
         return df_widget
 
@@ -630,14 +638,19 @@ def _hv_link_dataframes(fig, dfs):
 
     def scroll_table(tables):
         def record_taps(x, y):
-            for table in tables:
-                if x is not None:
-                    df = table.value
-                    i = df.index.get_loc(x, method='ffill')
-                    # On the pn.widgets.DataFrame, this will automatically scroll in the table.
-                    # On pn.widgets.Tabulator, this will currently not unfortunately
-                    table.selection = [i]
-            return hv.Points([])
+            try:
+                for table in tables:
+                    if x is not None:
+                        df = table.value
+                        i = df.index.get_indexer([x], method='ffill')[0]
+                        # This will automatically scroll in the table.
+                        # It requires a Python int, a numpy object is not good
+                        # enough.
+                        table.selection = [int(i)]
+            # It is vital to return something, otherwise the plot will
+            # disappear, which is much worse than the selection not working
+            finally:
+                return hv.Points([])
 
         tap = hv.streams.SingleTap(transient=True)
         dmap = hv.DynamicMap(record_taps, streams=[tap])
@@ -665,11 +678,11 @@ def _hv_link_dataframes(fig, dfs):
                 (table.name, table)
                 for table in tables
             ),
-            align='center',
+            align='start',
         )
     else:
         tables_widget = tables[0]
-        tables_widget.align = 'center'
+        tables_widget.align = 'start'
 
     return pn.Column(
         fig,
@@ -684,7 +697,6 @@ class _HoloviewsPanelWrapper:
     Dummy base class used to identify classes created by
     :func:`_hv_wrap_fig_cls`.
     """
-    pass
 
 @functools.lru_cache(maxsize=None)
 def _hv_wrap_fig_cls(cls):
