@@ -33,7 +33,6 @@ from bokeh.models import HoverTool
 
 from lisa.utils import Loggable, memoized, FrozenDict, deduplicate, fold
 from lisa.datautils import df_split_signals, df_make_empty_clone, df_filter, df_find_redundant_cols
-from lisa.notebook import make_figure, COLOR_CYCLE
 
 # Ensure hv.extension() is called
 import lisa.notebook
@@ -403,7 +402,7 @@ class Stats(Loggable):
 
         # Check that tags are sufficient to describe the data, so that we don't
         # end up with 2 different values for the same set of tags
-        duplicated_tags_size = df.groupby(tag_cols, observed=True).size()
+        duplicated_tags_size = df.groupby(tag_cols, observed=True, group_keys=False).size()
         duplicated_tags_size = duplicated_tags_size[duplicated_tags_size > 1]
         if not duplicated_tags_size.empty:
             raise ValueError(f'Same tags applied to more than one value, another tag column is needed to distinguish them:\n{duplicated_tags_size}')
@@ -729,7 +728,7 @@ class Stats(Loggable):
             series_len = len(series)
             if series_len < min_sample_size:
                 group_str = ', '.join(sorted(f'{k}={v}' for k, v in group.items()))
-                self.get_logger().warning(f'Sample size smaller than {min_sample_size} is being used, the mean confidence interval will only be accurate if the data is normally distributed: {series_len} samples for group {group_str}')
+                self.logger.warning(f'Sample size smaller than {min_sample_size} is being used, the mean confidence interval will only be accurate if the data is normally distributed: {series_len} samples for group {group_str}')
 
             mean, std, sem, ci = series_mean_stats(series, kind=mean_kind, confidence_level=self._mean_ci_confidence)
 
@@ -783,7 +782,7 @@ class Stats(Loggable):
             for name, func in stats.items()
         }
         if funcs:
-            grouped = df.groupby(tag_cols, observed=True, sort=False)
+            grouped = df.groupby(tag_cols, observed=True, sort=False, group_keys=False)
             df = grouped[self._val_col].agg(**funcs).reset_index()
             # Transform the newly created stats columns into rows
             df = self._melt(df)
@@ -828,7 +827,7 @@ class Stats(Loggable):
         test_df[self._unit_col] = 'pval'
         test_df = self._df_remove_tweak_cols(test_df)
 
-        return df.append(test_df, ignore_index=True)
+        return pd.concat([df, test_df], ignore_index=True)
 
     @_needs_ref
     def _df_compare_pct(self, df):
@@ -926,7 +925,7 @@ class Stats(Loggable):
 
         subplots = dict(
             plot_subdf(group, subdf)
-            for group, subdf in df.groupby(group_on, observed=True)
+            for group, subdf in df.groupby(group_on, observed=True, group_keys=False)
         )
 
         kdims = sorted(set(itertools.chain.from_iterable(
@@ -952,9 +951,6 @@ class Stats(Loggable):
             ],
             kdims=kdims,
         ).cols(ncols).options(
-            title=title,
-            shared_axes=False,
-        ).options(
             backend='bokeh',
             toolbar='left',
         ).options(
@@ -968,6 +964,9 @@ class Stats(Loggable):
             'Overlay',
             backend='bokeh',
             hooks=[lisa.notebook._hv_multi_line_title_hook],
+        ).options(
+            title=title,
+            shared_axes=False,
         )
 
         if filename:
