@@ -18,6 +18,7 @@
 from datetime import datetime
 import os
 import os.path
+from pathlib import Path
 import contextlib
 import shlex
 from collections.abc import Mapping
@@ -29,6 +30,8 @@ import functools
 import inspect
 import pickle
 import tempfile
+import hashlib
+import shutil
 from types import ModuleType, FunctionType
 from operator import itemgetter
 
@@ -37,14 +40,23 @@ from devlib.exception import TargetStableError
 from devlib.utils.misc import which
 from devlib.platform.gem5 import Gem5SimulationPlatform
 
-from lisa.utils import Loggable, HideExekallID, resolve_dotted_name, get_subclasses, import_all_submodules, LISA_HOME, RESULT_DIR, LATEST_LINK, setup_logging, ArtifactPath, nullcontext, ExekallTaggable, memoized
+from lisa.utils import Loggable, HideExekallID, resolve_dotted_name, get_subclasses, import_all_submodules, LISA_HOME, RESULT_DIR, LATEST_LINK, setup_logging, ArtifactPath, nullcontext, ExekallTaggable, memoized, destroyablecontextmanager, ContextManagerExit
 from lisa._assets import ASSETS_PATH
 from lisa.conf import SimpleMultiSrcConf, KeyDesc, LevelKeyDesc, TopLevelKeyDesc,Configurable
-from lisa._generic import TypedList
+from lisa._generic import TypedList, TypedDict
+from lisa._kmod import KernelTree, DynamicKmod
 
 from lisa.platforms.platinfo import PlatformInfo
 
 
+try:
+    import devlib.utils.asyn
+except ImportError:
+    _DEVLIB_HAS_ASYNC = False
+else:
+    _DEVLIB_HAS_ASYNC = True
+
+
 class PasswordKeyDesc(KeyDesc):
     def pretty_format(self, v):
         return '<password>'
@@ -145,6 +157,14 @@ class TargetConf(SimpleMultiSrcConf, HideExekallID):
         KeyDesc('workdir', 'Remote target workdir', [str]),
         KeyDesc('tools', 'List of tools to install on the target', [TypedList[str]]),
         KeyDesc('lazy-platinfo', 'Lazily autodect the platform information to speed up the connection', [bool]),
+        LevelKeyDesc('kernel', 'kernel information', (
+            KeyDesc('src', 'Path to kernel source tree matching the kernel running on the target used to build modules', [str, None]),
+            LevelKeyDesc('modules', 'kernel modules', (
+                KeyDesc('build-env', 'Environment used to build modules. Can be any of "alpine" (Alpine Linux chroot, recommended) or "host" (host system)', [str]),
+                KeyDesc('make-variables', 'Extra variables to pass to "make" command, such as "CC"', [TypedDict[str, object]]),
+                KeyDesc('overlay-backend', 'Backend to use for overlaying folders while building modules. Can be "overlayfs" (overlayfs filesystem, recommended) or "copy (plain folder copy)', [str]),
+            )),
+        )),
         LevelKeyDesc('wait-boot', 'Wait for the target to finish booting', (
             KeyDesc('enable', 'Enable the boot check', [bool]),
             KeyDesc('timeout', 'Timeout of the boot check', [int]),
@@ -159,6 +179,7 @@ class TargetConf(SimpleMultiSrcConf, HideExekallID):
             )),
             KeyDesc('excluded-modules', 'List of devlib modules to *not* load', [TypedList[str]]),
             KeyDesc('file-xfer', 'File transfer method. Can be "sftp" (default) or "scp". (Only valid for linux targets)', [TypedList[str]]),
+            KeyDesc('max-async', 'Maximum number of asynchronous commands in flight at any time', [int, None]),
 
         ))
     ))
@@ -234,22 +255,37 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
     INIT_KWARGS_KEY_MAP = {
         'devlib_excluded_modules': ['devlib', 'excluded-modules'],
         'devlib_file_xfer': ['devlib', 'file-xfer'],
+        'devlib_max_async': ['devlib', 'max-async'],
         'wait_boot': ['wait-boot', 'enable'],
         'wait_boot_timeout': ['wait-boot', 'timeout'],
+
+        'kernel_src': ['kernel', 'src'],
+        'kmod_build_env': ['kernel', 'modules', 'build-env'],
+        'kmod_make_vars': ['kernel', 'modules', 'make-variables'],
+        'kmod_overlay_backend': ['kernel', 'modules', 'overlay-backend'],
     }
 
     def __init__(self, kind, name='<noname>', tools=[], res_dir=None,
         plat_info=None, lazy_platinfo=False, workdir=None, device=None, host=None, port=None,
         username=None, password=None, keyfile=None, strict_host_check=None,
         devlib_platform=None, devlib_excluded_modules=[], devlib_file_xfer=None,
-        wait_boot=True, wait_boot_timeout=10,
+        wait_boot=True, wait_boot_timeout=10, kernel_src=None, kmod_build_env=None,
+        kmod_make_vars=None, kmod_overlay_backend=None, devlib_max_async=None,
     ):
         # pylint: disable=dangerous-default-value
         super().__init__()
-        logger = self.get_logger()
+        logger = self.logger
 
         self.name = name
 
+        self._kmod_tree = None
+        self._kmod_tree_spec = dict(
+            tree_path=kernel_src,
+            build_env=kmod_build_env,
+            make_vars=kmod_make_vars,
+            overlay_backend=kmod_overlay_backend,
+        )
+
         res_dir = res_dir if res_dir else self._get_res_dir(
             root=os.path.join(LISA_HOME, RESULT_DIR),
             relative='',
@@ -263,23 +299,6 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
         if os.listdir(self._res_dir):
             raise ValueError(f'res_dir must be empty: {self._res_dir}')
 
-        if plat_info is None:
-            plat_info = PlatformInfo()
-        else:
-            # Make a copy of the PlatformInfo so we don't modify the original
-            # one we were passed when adding the target source to it
-            plat_info = copy.copy(plat_info)
-            logger.info(f'User-defined platform information:\n{plat_info}')
-
-        self.plat_info = plat_info
-
-        # Take the board name from the target configuration so it becomes
-        # available for later inspection. That board name is mostly free form
-        # and no specific value should be expected for a given kind of board
-        # (i.e. a Juno board might be named "foo-bar-juno-on-my-desk")
-        if name:
-            self.plat_info.add_src('target-conf', dict(name=name))
-
         # Determine file transfer method. Currently avaliable options
         # are 'sftp' and 'scp', defaults to sftp.
         if devlib_file_xfer and devlib_file_xfer not in ('scp', 'sftp'):
@@ -302,6 +321,7 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
             devlib_platform=devlib_platform,
             wait_boot=wait_boot,
             wait_boot_timeout=wait_boot_timeout,
+            max_async=devlib_max_async,
         )
 
         devlib_excluded_modules = set(devlib_excluded_modules)
@@ -321,12 +341,114 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
         # Autodetect information from the target, after the Target is
         # initialized. Expensive computations are deferred so they will only be
         # computed when actually needed.
-
-        rta_calib_res_dir = ArtifactPath.join(self._res_dir, 'rta_calib')
-        os.makedirs(rta_calib_res_dir)
-        self.plat_info.add_target_src(self, rta_calib_res_dir, deferred=lazy_platinfo, fallback=True)
+        self._init_plat_info(plat_info, name, deferred=lazy_platinfo, fallback=True)
 
         logger.info(f'Effective platform information:\n{self.plat_info}')
+        cache_dir = Path(res_dir).resolve() / '.lisa' / 'cache'
+        cache_dir.mkdir(parents=True)
+        self._cache_dir = cache_dir
+
+    def _init_plat_info(self, plat_info=None, name=None, **kwargs):
+
+        if plat_info is None:
+            plat_info = PlatformInfo()
+        else:
+            # Make a copy of the PlatformInfo so we don't modify the original
+            # one we were passed when adding the target source to it
+            plat_info = copy.copy(plat_info)
+            self.logger.info(f'User-defined platform information:\n{plat_info}')
+
+        # Take the board name from the target configuration so it becomes
+        # available for later inspection. That board name is mostly free form
+        # and no specific value should be expected for a given kind of board
+        # (i.e. a Juno board might be named "foo-bar-juno-on-my-desk")
+        if name:
+            plat_info.add_src('target-conf', dict(name=name))
+
+        rta_calib_res_dir = ArtifactPath.join(self._res_dir, 'rta_calib')
+        os.makedirs(rta_calib_res_dir, exist_ok=True)
+        plat_info.add_target_src(self, rta_calib_res_dir, **kwargs)
+        self.plat_info = plat_info
+
+    def __getstate__(self):
+        return {
+            k: v
+            for k, v in self.__dict__.items()
+            if k not in {
+                # this KernelTree contains a reference to ourselves, and will
+                # fail to unpickle because of the circular dependency and the
+                # fact that it will call its constructor again, trying to make
+                # use of the Target before it is ready.
+                '_kmod_tree',
+            }
+        }
+
+    def __setstate__(self, dct):
+        self.__dict__.update(dct)
+        self._init_plat_info(deferred=True)
+        self._kmod_tree = None
+
+    def get_kmod(self, mod_cls=DynamicKmod, **kwargs):
+        """
+        Build a :class:`lisa._kmod.DynamicKmod` instance of the given
+        subclass.
+
+        :param mod_cls: Subclass of :class:`lisa._kmod.DynamicKmod` used to
+            build the module.
+        :type mod_cls: type
+
+        :Variable keyword arguments: Forwarded to the ``from_target`` class
+            method of ``mod_cls``.
+        """
+        try:
+            tree = kwargs['kernel_tree']
+        except KeyError:
+            memoize_tree = True
+            if self._kmod_tree:
+                tree = self._kmod_tree
+            else:
+                tree = KernelTree.from_target(
+                    target=self,
+                    **self._kmod_tree_spec
+                )
+        else:
+            memoize_tree = False
+
+        kwargs['kernel_tree'] = tree
+
+        mod = mod_cls.from_target(
+            target=self,
+            **kwargs,
+        )
+        if memoize_tree:
+            # Memoize the KernelTree, as it is a reusable object. Memoizing
+            # allows remembering its checksum across calls, which will allow
+            # hitting the .ko cache without having to setup a kernel tree.
+            self._kmod_tree = mod.kernel_tree
+        return mod
+
+    def cached_pull(self, src, dst, **kwargs):
+        """
+        Same as ``lisa.target.Target.pull`` but will cache the file in the
+        ``target.res_dir`` folder, based on the source path.
+
+        :Variable keyword arguments: Forwarded to ``Target.pull``.
+        """
+        cache = (self._cache_dir / 'pull')
+        cache.mkdir(parents=True, exist_ok=True)
+
+        m = hashlib.sha256()
+        m.update(src.encode('utf-8'))
+        key = m.hexdigest()
+        cached_path = cache / key / os.path.basename(src)
+
+        if not cached_path.exists():
+            self.pull(src, cached_path, **kwargs)
+
+        if cached_path.is_dir():
+            shutil.copytree(cached_path, dst)
+        else:
+            shutil.copy2(cached_path, dst)
 
     @property
     @memoized
@@ -372,6 +494,11 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
 
         .. note:: Devlib modules are loaded on demand when accessed.
         """
+        if (
+            attr in ('_devlib_loadable_modules', 'target') or
+            (attr.startswith('__') and attr.endswith('__'))
+        ):
+            raise AttributeError(attr)
 
         def get():
             return getattr(self.target, attr)
@@ -381,7 +508,7 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
         except AttributeError:
             # Load the module on demand
             if attr in self._devlib_loadable_modules:
-                self.get_logger().info(f'Loading target devlib module {attr}')
+                self.logger.info(f'Loading target devlib module {attr}')
                 self.target.install_module(attr)
                 return get()
             # If it was not in the loadable list, it
@@ -472,7 +599,7 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
         return target
 
     @classmethod
-    def from_custom_cli(cls, argv=None, params=None):
+    def from_custom_cli(cls, argv=None, params=None, description=None):
         """
         Create a Target from command line arguments.
 
@@ -485,14 +612,18 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
             ``{param_name: {dict of ArgumentParser.add_argument() options}}``.
         :type params: dict(str, dict)
 
+        :param description: Description passed to the argument parser. If
+            ``None``, a default one is provided.
+        :type description: str or None
+
         :return: A tuple ``(args, target)``
 
         .. note:: This method should not be relied upon to implement long-term
             scripts, it's more designed for quick scripting.
         """
-        parser = argparse.ArgumentParser(
-            formatter_class=argparse.RawDescriptionHelpFormatter,
-            description=textwrap.dedent(
+
+        if description is None:
+            description = textwrap.dedent(
                 """
                 Connect to a target using the provided configuration in order
                 to run a test.
@@ -512,7 +643,13 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
                 code execution.
                 """.format(
                     script=os.path.basename(sys.argv[0])
-                )))
+                )
+            )
+
+        parser = argparse.ArgumentParser(
+            formatter_class=argparse.RawDescriptionHelpFormatter,
+            description=description,
+        )
 
         parser.add_argument("--conf", '-c',
             help="Path to a TargetConf and PlatformInfo yaml file. Other options will override what is specified in the file."
@@ -543,7 +680,12 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
 
         params = params or {}
         for param, settings in params.items():
-            parser.add_argument(f'--{param}', **settings)
+            if settings.get('nargs') == argparse.REMAINDER:
+                _param = param
+            else:
+                _param = f'--{param}'
+            parser.add_argument(_param, **settings)
+
         custom_params = {k.replace('-', '_') for k in params.keys()}
 
         # Options that are not a key in TargetConf must be listed here
@@ -601,13 +743,12 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
 
     def _init_target(self, kind, name, workdir, device, host,
             port, username, password, keyfile, strict_host_check, use_scp,
-            devlib_platform,
-            wait_boot, wait_boot_timeout,
+            devlib_platform, wait_boot, wait_boot_timeout, max_async,
     ):
         """
         Initialize the Target
         """
-        logger = self.get_logger()
+        logger = self.logger
         conn_settings = {}
         resolved_username = username or 'root'
 
@@ -678,7 +819,8 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
         # Create devlib Target object
         ########################################################################
 
-        target = devlib_target_cls(
+        max_async = 50 if max_async is None else max_async
+        devlib_kwargs = dict(
             platform=devlib_platform,
             load_default_modules=False,
             connection_settings=conn_settings,
@@ -686,20 +828,26 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
             connect=False,
         )
 
-        target.connect(check_boot_completed=wait_boot, timeout=wait_boot_timeout)
+        connect_kwargs = dict(
+            check_boot_completed=wait_boot,
+            timeout=wait_boot_timeout,
+        )
 
-        # None as username means adb root will be attempted, but failure will
-        # not prevent from connecting to the target.
-        if kind == 'android' and username is None:
-            try:
-                target.adb_root(enable=True)
-            except Exception as e: # pylint: disable=broad-except
-                logger.warning(f'"adb root" failed: {e}')
+        if _DEVLIB_HAS_ASYNC:
+            devlib_kwargs['max_async'] = max_async
+            connect_kwargs['max_async'] = max_async
 
+        devlib_kwargs = {
+            k: v
+            for k, v in devlib_kwargs.items()
+            if v is not None
+        }
+
+        target = devlib_target_cls(**devlib_kwargs)
+
+        target.connect(**connect_kwargs)
         logger.debug(f'Target info: {dict(abi=target.abi, cpuinfo=target.cpuinfo, workdir=target.working_directory)}')
-
         target.setup()
-
         logger.info(f"Connected to target {(name or '')}")
         return target
 
@@ -741,7 +889,7 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
         )
 
     def _get_res_dir(self, root, relative, name, append_time, symlink):
-        logger = self.get_logger()
+        logger = self.logger
         while True:
             time_str = datetime.now().strftime('%Y%m%d_%H%M%S.%f')
             if not name:
@@ -797,16 +945,14 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
                 binary = os.path.join(ASSETS_PATH, 'binaries', 'scripts', tool)
             return binary
 
-        tools = set(tools) - self._installed_tools
+        tools = sorted(set(tools) - self._installed_tools)
 
         # TODO: compute the checksum of the tool + install location and keep
         # that in _installed_tools, so we are sure to be correct
-        for tool in map(bin_path, tools):
-            self.target.install(tool)
+        for tool in tools:
+            self.target.install(bin_path(tool))
             self._installed_tools.add(tool)
 
-
-    @contextlib.contextmanager
     def freeze_userspace(self):
         """
         Context manager that lets you freeze the userspace.
@@ -815,7 +961,7 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
             that happens, a warning is logged but no exception is raised, so
             it's a best-effort approach.
         """
-        logger = self.get_logger()
+        logger = self.logger
         if not self.is_rooted:
             logger.warning('Could not freeze userspace: target is not rooted')
             cm = nullcontext
@@ -836,24 +982,22 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
                 if isinstance(self.target, devlib.LocalLinuxTarget):
                     exclude.append(str(os.getpid()))
 
-                @contextlib.contextmanager
+                @destroyablecontextmanager
                 def cm():
                     logger.info(f"Freezing all tasks except: {','.join(exclude)}")
                     try:
                         yield self.cgroups.freeze(exclude)
-                    finally:
+                    except ContextManagerExit:
                         logger.info('Un-freezing userspace tasks')
                         self.cgroups.freeze(thaw=True)
 
-        with cm() as x:
-            yield x
+        return cm()
 
-    @contextlib.contextmanager
     def disable_idle_states(self):
         """
         Context manager that lets you disable all idle states
         """
-        logger = self.get_logger()
+        logger = self.logger
         logger.info('Disabling idle states for all domains')
 
         try:
@@ -862,19 +1006,18 @@ class Target(Loggable, HideExekallID, ExekallTaggable, Configurable):
             logger.warning('Could not disable idle states, cpuidle devlib module is not loaded')
             cm = nullcontext
         else:
-            @contextlib.contextmanager
+            @destroyablecontextmanager
             def cm():
                 try:
                     for cpu in range(self.plat_info['cpus-count']):
                         cpuidle.disable_all(cpu)
                     yield
-                finally:
+                except ContextManagerExit:
                     logger.info('Re-enabling idle states for all domains')
                     for cpu in range(self.plat_info['cpus-count']):
                         cpuidle.enable_all(cpu)
 
-        with cm() as x:
-            yield x
+        return cm()
 
     def get_tags(self):
         return {'board': self.name}
