@@ -30,7 +30,7 @@ import contextlib
 import itertools
 import types
 import warnings
-from operator import attrgetter, itemgetter
+from operator import attrgetter
 
 from datetime import datetime
 from collections import OrderedDict, ChainMap
@@ -44,17 +44,17 @@ from devlib import TargetStableError
 
 from lisa.analysis.tasks import TasksAnalysis
 from lisa.analysis.rta import RTAEventsAnalysis
-from lisa.trace import requires_events, may_use_events
+from lisa.trace import requires_events, TraceEventCheckerBase, AndTraceEventChecker
 from lisa.trace import Trace, TaskID
-from lisa.wlgen.rta import RTA, PeriodicWload, RTAPhase
+from lisa.wlgen.rta import RTA, PeriodicWload, RTAPhase, leaf_precedence
 from lisa.target import Target
 
 from lisa.utils import (
     Serializable, memoized, lru_memoized, ArtifactPath, non_recursive_property,
     update_wrapper_doc, ExekallTaggable, annotations_from_signature,
-    nullcontext, get_sphinx_name, optional_kwargs, group_by_value,
-    kwargs_dispatcher, dispatch_kwargs, Loggable, kwargs_forwarded_to,
-    docstring_update, is_running_ipython,
+    get_sphinx_name, optional_kwargs, group_by_value, kwargs_dispatcher,
+    dispatch_kwargs, Loggable, kwargs_forwarded_to, docstring_update,
+    is_running_ipython,
 )
 from lisa.datautils import df_filter_task_ids
 from lisa.trace import FtraceCollector, FtraceConf, DmesgCollector, ComposedCollector
@@ -62,7 +62,7 @@ from lisa.conf import (
     SimpleMultiSrcConf, KeyDesc, TopLevelKeyDesc,
 )
 from lisa._generic import TypedList
-from lisa.pelt import PELT_SCALE, pelt_settling_time
+from lisa.pelt import pelt_settling_time
 
 
 def _nested_formatter(multiline):
@@ -1008,7 +1008,6 @@ class TestBundleBase(
                 def _from_target(cls, target, *, foo=33, bar):
                     ...
         """
-        pass
 
     @classmethod
     def check_from_target(cls, target):
@@ -1020,7 +1019,6 @@ class TestBundleBase(
 
         This method should be overriden to check your implementation requirements
         """
-        pass
 
     @classmethod
     def can_create_from_target(cls, target):
@@ -1211,9 +1209,9 @@ class TestBundleBase(
         super().to_path(self._get_filepath(res_dir))
 
 
-class FtraceTestBundle(TestBundleBase):
+class FtraceTestBundleBase(TestBundleBase):
     """
-    Abstract Base Class for :class:`lisa.wlgen.rta.RTA`-powered TestBundles
+    Base class for test bundles needing ftrace traces.
 
     Optionally, an ``FTRACE_CONF`` class attribute can be defined to hold
     additional FTrace configuration used to record a trace while the synthetic
@@ -1252,14 +1250,16 @@ class FtraceTestBundle(TestBundleBase):
 
         # Collect all the events that can be used by all methods available on
         # that class.
-        ftrace_events = set()
+        ftrace_events = []
         for name, obj in inspect.getmembers(cls, callable):
             try:
                 used_events = obj.used_events
             except AttributeError:
                 continue
             else:
-                ftrace_events.update(used_events.get_all_events())
+                ftrace_events.append(used_events)
+
+        ftrace_events = AndTraceEventChecker(ftrace_events)
 
         # Get the ftrace_conf attribute of the class, and make sure it is
         # unique to that class (i.e. not shared with any other parent or
@@ -1289,7 +1289,7 @@ class FtraceTestBundle(TestBundleBase):
         ftrace_conf.add_merged_src(
             src=f'{cls.__qualname__}(required)',
             conf={
-                'events': sorted(ftrace_events),
+                'events': ftrace_events,
             },
         )
 
@@ -1312,6 +1312,7 @@ class FtraceTestBundle(TestBundleBase):
         conf.add_merged_src(
             src=f'user+{cls.__qualname__}',
             conf=user_conf,
+            optional_events=True,
         )
 
         # If there is no event, do not collect the trace unless the user asked
@@ -1338,7 +1339,12 @@ class FtraceTestBundle(TestBundleBase):
     # Guard before the cache, so we don't accidentally start depending on the
     # LRU cache for functionnal correctness.
     @non_recursive_property
-    @memoized
+    # Only cache the trace of N bundles at a time, to avoid running out of memory.
+    # This should not really impact the test when ran with exekall, since they
+    # are sequenced one after another. It would have some speed impact on
+    # scripts/notebooks that try to do something with a bunch of
+    # FtraceTestBundle.
+    @lru_memoized(first_param_maxsize=5)
     def trace(self):
         """
         :returns: a :class:`lisa.trace.TraceView`
@@ -1352,15 +1358,49 @@ class FtraceTestBundle(TestBundleBase):
         allows updating the underlying path before it is actually loaded to
         match a different folder structure.
         """
-        return self.get_trace(events=self.FTRACE_CONF["events"], normalize_time=True)
+        return self.get_trace(
+            events=self.FTRACE_CONF["events"],
+            normalize_time=True,
+            # Soft limit on the amount of memory used by dataframes kept around
+            # in memory by Trace, so that we don't blow up the memory when we
+            # have a large-ish number of FTraceTestBundle alive at the same
+            # time.
+            max_mem_size=500e6,
+            # TODO: revisit that. As of pyarrow 2.0.0 and pandas 1.1.4, reading
+            # (and maybe writing) parquet fils seem to leak memory. This can
+            # take the consumption in the order of tens of gigabytes for a few
+            # iterations of the tests with exekall, leading to crashes.
+            # Therefore, disable the on-disk swap.
+            enable_swap=False,
+        )
 
-    def get_trace(self, **kwargs):
+    def get_trace(self, events=None, **kwargs):
         """
         :returns: a :class:`lisa.trace.Trace` collected in the standard location.
 
         :Variable keyword arguments: Forwarded to :class:`lisa.trace.Trace`.
         """
-        return Trace(self.trace_path, self.plat_info, **kwargs)
+        return Trace(self.trace_path, self.plat_info, events=events, **kwargs)
+
+
+class FtraceTestBundle(FtraceTestBundleBase):
+    """
+    Dummy subclass of :class:`FtraceTestBundleBase` to be inherited from to
+    override :class:`OptionalFtraceTestBundle` in the inheritance tree.
+    """
+    _make_ftrace_collector = FtraceTestBundleBase._make_ftrace_collector
+
+
+class OptionalFtraceTestBundle(FtraceTestBundleBase, Loggable):
+    @classmethod
+    @TestBundleBase.collector_factory
+    @kwargs_forwarded_to(FtraceTestBundleBase._make_ftrace_collector)
+    def _make_ftrace_collector(cls, **kwargs):
+        try:
+            return super()._make_ftrace_collector(**kwargs)
+        except Exception as e:
+            cls.get_logger().warning(f'Could not create ftrace collector: {e}')
+            return None
 
 
 class TestConfBase(SimpleMultiSrcConf):
@@ -1386,7 +1426,7 @@ class DmesgTestConf(TestConfBase):
     {yaml_example}
     """
     STRUCTURE = TopLevelKeyDesc('dmesg', 'Dmesg test configuration', (
-        KeyDesc('ignored-patterns', 'List of Python regex matching dmesg entries *content* to be whitelisted (see :class:`devlib.collector.dmesg.KernelLogEntry` for how the message is split)', [TypedList[str]]),
+        KeyDesc('ignored-patterns', 'List of Python regex matching dmesg entries *content* to be ignored (see :class:`devlib.collector.dmesg.KernelLogEntry` for how the message is split)', [TypedList[str]]),
     ))
 
 
@@ -1408,8 +1448,6 @@ class DmesgTestBundleBase(TestBundleBase):
         # On kernel >= 5.6, executable stack will trigger this issue:
     	# kern: warn: [555.927466] process 'root/devlib-target/bin/busybox' started with executable stack
         'executable-stack': 'started with executable stack',
-        'hotplug-irq-affinity': 'no longer affine to',
-        'hotplug-irq-affinity-failed': 'set affinity failed',
     }
     """
     Mapping of canned patterns to avoid repetition while defining
@@ -1478,7 +1516,7 @@ class DmesgTestBundleBase(TestBundleBase):
             (self.DMESG_IGNORED_PATTERNS or [])
         )
 
-        logger = self.get_logger()
+        logger = self.logger
 
         if ignored_patterns:
             logger.info(f'Will ignore patterns in dmesg output: {ignored_patterns}')
@@ -1521,7 +1559,7 @@ class OptionalDmesgTestBundle(DmesgTestBundleBase, Loggable):
         try:
             return super().test_dmesg(*args, **kwargs)
         except FileNotFoundError:
-            self.get_logger().warning('Could not check dmesg content, as it was not collected')
+            self.logger.warning('Could not check dmesg content, as it was not collected')
             return ResultBundle(result=Result.UNDECIDED)
 
     @classmethod
@@ -1539,6 +1577,11 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
     """
     Abstract Base Class for :class:`lisa.wlgen.rta.RTA`-powered TestBundles
 
+    :param rtapp_profile_kwargs: Keyword arguments to pass to
+        :meth:`lisa.tests.base.RTATestBundle._get_rtapp_profile` when called from
+        the :meth:`lisa.tests.base.RTATestBundle._get_rtapp_profile` property.
+    :type rtapp_profile_kwargs: collections.abc.Mapping or None
+
     .. seealso: :class:`lisa.tests.base.FtraceTestBundle` for default
         ``FTRACE_CONF`` content.
     """
@@ -1584,6 +1627,10 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
     Properties of the buffer phase, see :attr:`_BUFFER_PHASE_DURATION_S`
     """
 
+    def __init__(self, res_dir, plat_info, rtapp_profile_kwargs=None):
+        super().__init__(res_dir, plat_info)
+        self._rtapp_profile_kwargs = dict(rtapp_profile_kwargs or {})
+
     @RTAEventsAnalysis.df_rtapp_phases_start.used_events
     @RTAEventsAnalysis.df_rtapp_phases_end.used_events
     @requires_events('sched_switch')
@@ -1646,7 +1693,11 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
         """
         Compute the RTapp profile based on ``plat_info``.
         """
-        return self.get_rtapp_profile(self.plat_info)
+
+        return self.get_rtapp_profile(
+            self.plat_info,
+            **self._rtapp_profile_kwargs,
+        )
 
     _rtapp_tasks_events = requires_events('sched_switch')
 
@@ -1707,41 +1758,15 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
         return self.get_cgroup_configuration(self.plat_info)
 
     @non_recursive_property
-    # Only cache the trace of N bundles at a time, to avoid running out of memory.
-    # This should not really impact the test when ran with exekall, since they
-    # are sequenced one after another. It would have some speed impact on
-    # scripts/notebooks that try to do something with a bunch of
-    # FtraceTestBundle.
     @lru_memoized(first_param_maxsize=5)
     def trace(self):
         """
         :returns: a :class:`lisa.trace.TraceView` cropped to the window given
             by :meth:`trace_window`.
 
-        All events specified in ``FTRACE_CONF`` are parsed from the trace,
-        so it is suitable for direct use in methods.
-
-        Having the trace as a property lets us defer the loading of the actual
-        trace to when it is first used. Also, this prevents it from being
-        serialized when calling :meth:`lisa.utils.Serializable.to_path` and
-        allows updating the underlying path before it is actually loaded to
-        match a different folder structure.
+        .. seealso:: :attr:`FtraceTestBundleBase.trace`
         """
-        trace = self.get_trace(
-            events=self.FTRACE_CONF["events"],
-            normalize_time=True,
-            # Soft limit on the amount of memory used by dataframes kept around
-            # in memory by Trace, so that we don't blow up the memory when we
-            # have a large-ish number of FTraceTestBundle alive at the same
-            # time.
-            max_mem_size=500e6,
-            # TODO: revisit that. As of pyarrow 2.0.0 and pandas 1.1.4, reading
-            # (and maybe writing) parquet fils seem to leak memory. This can
-            # take the consumption in the order of tens of gigabytes for a few
-            # iterations of the tests with exekall, leading to crashes.
-            # Therefore, disable the on-disk swap.
-            enable_swap=False,
-        )
+        trace = super().trace
         return trace.get_view(self.trace_window(trace), clear_base_cache=True)
 
     def df_noisy_tasks(self, with_threshold_exclusion=True):
@@ -1781,7 +1806,7 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
                  ignored_ids.extend(task_ids)
 
 
-        self.get_logger().info(f"Ignored PIDs for noise contribution: {', '.join(map(str, ignored_ids))}")
+        self.logger.info(f"Ignored PIDs for noise contribution: {', '.join(map(str, ignored_ids))}")
 
         # Filter out unwanted tasks (rt-app tasks + thresholds)
         df = df_filter_task_ids(df, ignored_ids, invert=True)
@@ -1880,7 +1905,7 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
         def add_buffer(task):
             template_phase = task.phases[0]
             wload = template_phase['wload']
-            task = task.with_props(meta={'from_test': True})
+            task = task.with_props(meta=leaf_precedence({'from_test': True}))
             if 'name' not in task:
                 task = task.with_props(name='test')
 
@@ -1934,7 +1959,6 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
         This is the method you want to override to specify what is your
         synthetic workload.
         """
-        pass
 
     @classmethod
     def get_cgroup_configuration(cls, plat_info):
@@ -2031,14 +2055,6 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
                 if event.startswith('userspace@rtapp_')
             ]
 
-        # Coarse-grained detection, but that should be enough for our use
-        try:
-            target.execute('ls /sys/kernel/debug/tracing/')
-        except TargetStableError:
-            debugfs_needs_root = True
-        else:
-            debugfs_needs_root = False
-
         wload = RTA.from_profile(
             target=target,
             profile=profile,
@@ -2048,6 +2064,12 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
             # Force the default value for all settings so that the test does
             # not depend on the environment setup.
             force_defaults=True,
+            no_force_default_keys=[
+                # Since "taskgroup" cannot be always expected to work in case
+                # cgroupfs is not mounted at all, we will not force a default
+                # value for it.
+                'taskgroup'
+            ],
         )
 
         profile_str = '\n'.join(
@@ -2061,7 +2083,9 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
         logger.info(f'rt-app workload:\n{profile_str}')
         logger.debug(f'rt-app JSON:\n{wload.conf.json}')
         cgroup = cls._target_configure_cgroup(target, cg_cfg)
-        as_root = cgroup is not None or (trace_events and debugfs_needs_root)
+        as_root = bool(
+            cgroup is not None or trace_events
+        )
 
         wload = wload(
             wipe_run_dir=wipe_run_dir,
@@ -2100,7 +2124,7 @@ class RTATestBundle(FtraceTestBundle, DmesgTestBundle):
         return cls(res_dir, plat_info)
 
 
-class TestBundle(FtraceTestBundle, OptionalDmesgTestBundle, TestBundleBase):
+class TestBundle(OptionalFtraceTestBundle, OptionalDmesgTestBundle, TestBundleBase):
     """
     Dummy class used as a base class for all tests.
     """
