@@ -18,7 +18,6 @@
 import abc
 import sys
 import random
-import os.path
 import operator
 import collections
 import time
@@ -29,10 +28,10 @@ from itertools import chain
 
 import pandas as pd
 from devlib.module.hotplug import HotplugModule
-from devlib.exception import TargetNotRespondingError, TargetStableError
+from devlib.exception import TargetStableError
 
 from lisa.datautils import df_merge
-from lisa.tests.base import TestMetric, ResultBundle, TestBundle, DmesgTestBundle, FtraceTestBundle
+from lisa.tests.base import DmesgTestBundle, FtraceTestBundle, ResultBundle, TestBundle
 from lisa.target import Target
 from lisa.trace import requires_events
 from lisa.utils import ArtifactPath
@@ -45,8 +44,7 @@ class CPUHPSequenceError(Exception):
 class HotplugDmesgTestBundle(DmesgTestBundle):
     DMESG_IGNORED_PATTERNS = [
         *DmesgTestBundle.DMESG_IGNORED_PATTERNS,
-        DmesgTestBundle.CANNED_DMESG_IGNORED_PATTERNS['hotplug-irq-affinity'],
-        DmesgTestBundle.CANNED_DMESG_IGNORED_PATTERNS['hotplug-irq-affinity-failed'],
+        'irq|IRQ',
     ]
 
 
@@ -115,7 +113,6 @@ class HotplugBase(HotplugDmesgTestBundle, TestBundle):
         plugged-off before (and vice versa). Moreover the state of the CPUs
         once the sequence has completed should the same as it was before.
         """
-        pass
 
     @classmethod
     def _cpuhp_func(cls, target, res_dir, sequence, sleep_min_ms,
@@ -278,7 +275,7 @@ class HotplugTorture(HotplugBase):
             yield cpu, 1
 
 
-class HotplugRollback(TestBundle, HotplugDmesgTestBundle, FtraceTestBundle):
+class HotplugRollback(HotplugDmesgTestBundle, FtraceTestBundle, TestBundle):
 
     @classmethod
     def _online(cls, target, cpu, online, verify=True):
@@ -474,8 +471,11 @@ class HotplugRollback(TestBundle, HotplugDmesgTestBundle, FtraceTestBundle):
 
         df['failing_state'].ffill(inplace=True)
         df.dropna(inplace=True)
-        df = df.groupby(['up', 'failing_state'],
-                        observed=True).apply(_verify_rollback)
+        df = df.groupby(
+            ['up', 'failing_state'],
+            observed=True,
+            group_keys=False,
+        ).apply(_verify_rollback)
         df.drop_duplicates(inplace=True)
 
         res = ResultBundle.from_bool(df['result'].all())
