@@ -15,27 +15,41 @@
 # limitations under the License.
 #
 
-import os.path
+import abc
 from math import isnan
 
 import pandas as pd
 import holoviews as hv
 
 from itertools import chain
-from devlib.target import KernelVersion
 
 from lisa.wlgen.rta import RTAPhase, PeriodicWload, DutyCycleSweepPhase
 from lisa.analysis.rta import RTAEventsAnalysis
 from lisa.analysis.tasks import TasksAnalysis
-from lisa.tests.base import ResultBundle, TestBundle, RTATestBundle
+from lisa.tests.base import ResultBundle, TestBundle, RTATestBundle, TestConfBase
 from lisa.utils import ArtifactPath, memoized
 from lisa.datautils import series_integrate, df_deduplicate
 from lisa.energy_model import EnergyModel, EnergyModelCapacityError
-from lisa.trace import requires_events
 from lisa.target import Target
 from lisa.pelt import PELT_SCALE, pelt_swing
 from lisa.datautils import df_refit_index
 from lisa.notebook import plot_signal
+from lisa.conf import (
+    KeyDesc, TopLevelKeyDesc,
+)
+
+
+class EASBehaviourTestConf(TestConfBase):
+    """
+    Configuration class for :meth:`lisa.tests.scheduler.eas_behaviour.EASBehaviour.get_big_duty_cycle`.
+
+    {generated_help}
+    {yaml_example}
+    """
+
+    STRUCTURE = TopLevelKeyDesc('eas-behaviour', 'EAS-behaviour test configuration', (
+        KeyDesc('big-task-duty-cycle', 'Duty cycle of the big tasks for the eas-behaviour tests.', [int]),
+    ))
 
 
 class EASBehaviour(RTATestBundle, TestBundle):
@@ -58,7 +72,15 @@ class EASBehaviour(RTATestBundle, TestBundle):
         return self.plat_info['nrg-model']
 
     @classmethod
-    def get_big_duty_cycle(cls, plat_info):
+    def get_pelt_swing(cls, pct):
+        return pelt_swing(
+            period=cls.TASK_PERIOD,
+            duty_cycle=pct / 100,
+            kind='above',
+        ) / PELT_SCALE * 100
+
+    @classmethod
+    def get_big_duty_cycle(cls, plat_info, big_task_duty_cycle=None):
         """
         Returns a duty cycle for :class:`lisa.wlgen.rta.PeriodicWload` that
         will guarantee placement on a big CPU.
@@ -67,45 +89,43 @@ class EASBehaviour(RTATestBundle, TestBundle):
         second to biggest CPUs in the system, thereby forcing up-migration
         while minimizing the thermal impact.
         """
-        capa_classes = plat_info['capacity-classes']
-        max_class = len(capa_classes) - 1
+        # big_task_duty_cycle is set when the platform requires a specific
+        # value for the big task duty cycle.
+        if big_task_duty_cycle is None:
+            capa_classes = plat_info['capacity-classes']
+            max_class = len(capa_classes) - 1
 
-        def get_class_util(class_, pct):
-            cpus = capa_classes[class_]
-            return cls.unscaled_utilization(plat_info, cpus[0], pct)
+            def get_class_util(class_, pct):
+                cpus = capa_classes[class_]
+                return cls.unscaled_utilization(plat_info, cpus[0], pct)
 
-        def get_pelt_swing(pct):
-            return pelt_swing(
-                period=cls.TASK_PERIOD,
-                duty_cycle=util / 100,
-                kind='above',
-            ) / PELT_SCALE * 100
+            class_ = -2
 
-        class_ = -2
+            # Resolve to an positive index
+            class_ %= (max_class + 1)
 
-        # Resolve to an positive index
-        class_ %= (max_class + 1)
+            capacity_margin_pct = 20
+            util = get_class_util(class_, 100)
 
-        capacity_margin_pct = 20
-        util = get_class_util(class_, 100)
-
-        if class_ < max_class:
-            higher_class_capa = get_class_util(class_ + 1, (100 - capacity_margin_pct))
-            # If the CPU class and util we picked is too close to the capacity
-            # of the next bigger CPU, we need to take a smaller util
-            if (util + get_pelt_swing(util)) >= higher_class_capa:
-                # Take a 5% margin for rounding errors
-                util = 0.95 * higher_class_capa
-                return (
-                    util -
-                    # And take extra margin to take into account the swing of
-                    # the PELT value around the average
-                    get_pelt_swing(util)
-                )
+            if class_ < max_class:
+                higher_class_capa = get_class_util(class_ + 1, (100 - capacity_margin_pct))
+                # If the CPU class and util we picked is too close to the capacity
+                # of the next bigger CPU, we need to take a smaller util
+                if (util + cls.get_pelt_swing(util)) >= higher_class_capa:
+                    # Take a 5% margin for rounding errors
+                    util = 0.95 * higher_class_capa
+                    return (
+                        util -
+                        # And take extra margin to take into account the swing of
+                        # the PELT value around the average
+                        cls.get_pelt_swing(util)
+                    )
+                else:
+                    return util
             else:
                 return util
         else:
-            return util
+            return big_task_duty_cycle
 
     @classmethod
     def get_little_cpu(cls, plat_info):
@@ -115,6 +135,63 @@ class EASBehaviour(RTATestBundle, TestBundle):
         littles = plat_info["capacity-classes"][0]
         return littles[0]
 
+    @classmethod
+    def get_little_duty_cycle(cls, plat_info):
+        """
+        Returns a duty cycle for :class:`lisa.wlgen.rta.PeriodicWload` that
+        is guaranteed to fit on the little CPUs.
+
+        The duty cycle is chosen to be ~50% of the capacity of the little CPU
+        and to generate a target frequency half-way between two frequencies of
+        that same CPU. This intends to avoid picking a value too close from an
+        OPP which could, for the same duty cycle use an upper OPP or not, depending
+        on the PELT hazard.
+
+        The returned value is a duty cycle in percentage of the full PELT scale.
+        """
+        cpu = cls.get_little_cpu(plat_info)
+        freqs = sorted(plat_info['freqs'][cpu])
+        capa = plat_info['cpu-capacities']['rtapp'][cpu]
+
+        max_freq = max(freqs)
+        target_freq = 0.5 * max_freq # 50% duty cycle
+        schedutil_factor = 1.25
+
+        # Return the PELT swing in pct for a given duty cycle in pct
+        def _get_pelt_swing_dc(dc):
+            return cls.get_pelt_swing(dc) * 100 / PELT_SCALE
+
+        # Duty cycle for a given frequency band
+        def _get_dc(freq_band):
+            minf, maxf = freq_band
+            freq = ((maxf - minf) / 2) + minf
+
+            # freq to dc in pct
+            dc = freq * 100 / max_freq * (capa / PELT_SCALE)
+
+            # Ensure that the max value of util_avg will more or less make
+            # schedutil select the midpoint in the freq_band
+            dc -= _get_pelt_swing_dc(dc)
+            dc /= schedutil_factor
+
+            # Check that the duty cycle we computed still fits in the selected
+            # frequency band
+            real_freq = (dc + _get_pelt_swing_dc(dc)) * schedutil_factor * \
+                max_freq / 100 * PELT_SCALE / capa
+
+            if minf < real_freq < maxf:
+                return dc
+            else:
+                raise ValueError(f'Could not find util fitting the frequency band {freq_band}')
+
+        minf, maxf = min(
+            (freq, next_freq)
+            for freq, next_freq in zip(freqs, freqs[1:])
+            if next_freq > target_freq
+        )
+
+        return _get_dc((minf, maxf))
+
     @classmethod
     def check_from_target(cls, target):
         super().check_from_target(target)
@@ -126,7 +203,7 @@ class EASBehaviour(RTATestBundle, TestBundle):
             if not kconfig.get(option):
                 ResultBundle.raise_skip(f"The target's kernel needs {option}=y kconfig enabled")
 
-        for domain in target.cpufreq.iter_domains():
+        for domain in target.plat_info['freq-domains']:
             if "schedutil" not in target.cpufreq.list_governors(domain[0]):
                 ResultBundle.raise_skip(
                     f"Can't set schedutil governor for domain {domain}")
@@ -135,7 +212,8 @@ class EASBehaviour(RTATestBundle, TestBundle):
             ResultBundle.raise_skip("Energy model not available")
 
     @classmethod
-    def _from_target(cls, target: Target, *, res_dir: ArtifactPath = None, collector=None) -> 'EASBehaviour':
+    def _from_target(cls, target: Target, *, res_dir: ArtifactPath = None, collector=None,
+            big_task_duty_cycle: EASBehaviourTestConf.BigTaskDutyCycle = None) -> 'EASBehaviour':
         """
         :meta public:
 
@@ -145,7 +223,9 @@ class EASBehaviour(RTATestBundle, TestBundle):
         :meth:`lisa.tests.base.RTATestBundle.get_rtapp_profile`
         """
         plat_info = target.plat_info
-        rtapp_profile = cls.get_rtapp_profile(plat_info)
+        profile_kwargs = dict(big_task_duty_cycle=big_task_duty_cycle)
+
+        rtapp_profile = cls.get_rtapp_profile(plat_info, **profile_kwargs)
 
         # EAS doesn't make a lot of sense without schedutil,
         # so make sure this is what's being used
@@ -153,7 +233,7 @@ class EASBehaviour(RTATestBundle, TestBundle):
             with target.cpufreq.use_governor("schedutil"):
                 cls.run_rtapp(target, res_dir, rtapp_profile, collector=collector)
 
-        return cls(res_dir, plat_info)
+        return cls(res_dir, plat_info, rtapp_profile_kwargs=profile_kwargs)
 
     @RTAEventsAnalysis.df_phases.used_events
     def _get_expected_task_utils_df(self):
@@ -496,7 +576,56 @@ class EASBehaviour(RTATestBundle, TestBundle):
         return res
 
 
-class OneSmallTask(EASBehaviour):
+class EASBehaviourNoEWMA(EASBehaviour):
+    """
+    Abstract class for EAS behavioural testing, with mitigation for the
+    util_est.ewma influence
+
+    This class provides :meth:`_get_rtapp_profile` which prepend a custom
+    RTAPhase buffer to the rtapp profile. This buffer is composed of a dozen
+    of very short activation. It intends to reset util_est.ewma before starting
+    the test. util_est.ewma is computed for the CFS policy on the utilization
+    ramp down. It holds the utilization value and prevents convergence to a
+    value matching the duty cycle set in the rt-app profile.
+    """
+
+    _BUFFER_PHASE_DURATION_S = 0 # Bypass add_buffer() default RTAPhase buffer
+
+    @abc.abstractmethod
+    def _do_get_rtapp_profile(cls, plat_info, **kwargs):
+        """
+        :meta public:
+
+        Abstract method used by children class to provide the rt-app profile
+        for the test to run.
+        """
+        pass
+
+    @classmethod
+    def _get_rtapp_profile(cls, plat_info, **kwargs):
+        """
+        :meta public:
+
+        Prepends a :class:`lisa.wlgen.rta.RTAPhase` buffer to the children
+        class rt-app profile :meth:`_do_get_rtapp_profile`. This buffer intends
+        to mitigate the util_est.ewma influence.
+        """
+        profile = cls._do_get_rtapp_profile(plat_info, **kwargs)
+
+        return {
+            task: RTAPhase(
+                prop_wload=PeriodicWload(
+                    duty_cycle_pct=0.01,
+                    duration=0.1,
+                    period=cls.TASK_PERIOD
+                ),
+                prop_meta={'from_test': False}
+            ) + phase
+            for task, phase in profile.items()
+        }
+
+
+class OneSmallTask(EASBehaviourNoEWMA):
     """
     A single 'small' task
     """
@@ -504,12 +633,11 @@ class OneSmallTask(EASBehaviour):
     task_name = "small"
 
     @classmethod
-    def _get_rtapp_profile(cls, plat_info):
+    def _do_get_rtapp_profile(cls, plat_info, **kwargs):
         return {
             cls.task_name: RTAPhase(
                 prop_wload=PeriodicWload(
-                    duty_cycle_pct=50,
-                    scale_for_cpu=cls.get_little_cpu(plat_info),
+                    duty_cycle_pct=cls.get_little_duty_cycle(plat_info),
                     duration=1,
                     period=cls.TASK_PERIOD,
                 )
@@ -517,7 +645,7 @@ class OneSmallTask(EASBehaviour):
         }
 
 
-class ThreeSmallTasks(EASBehaviour):
+class ThreeSmallTasks(EASBehaviourNoEWMA):
     """
     Three 'small' tasks
     """
@@ -545,12 +673,11 @@ class ThreeSmallTasks(EASBehaviour):
             capacity_margin_pct=capacity_margin_pct)
 
     @classmethod
-    def _get_rtapp_profile(cls, plat_info):
+    def _do_get_rtapp_profile(cls, plat_info, **kwargs):
         return {
             f"{cls.task_prefix}_{i}": RTAPhase(
                 prop_wload=PeriodicWload(
-                    duty_cycle_pct=50,
-                    scale_for_cpu=cls.get_little_cpu(plat_info),
+                    duty_cycle_pct=cls.get_little_duty_cycle(plat_info),
                     duration=1,
                     period=cls.TASK_PERIOD,
                 )
@@ -559,7 +686,7 @@ class ThreeSmallTasks(EASBehaviour):
         }
 
 
-class TwoBigTasks(EASBehaviour):
+class TwoBigTasks(EASBehaviourNoEWMA):
     """
     Two 'big' tasks
     """
@@ -567,8 +694,9 @@ class TwoBigTasks(EASBehaviour):
     task_prefix = "big"
 
     @classmethod
-    def _get_rtapp_profile(cls, plat_info):
-        duty = cls.get_big_duty_cycle(plat_info)
+    def _do_get_rtapp_profile(cls, plat_info, big_task_duty_cycle=None):
+        duty = cls.get_big_duty_cycle(plat_info, big_task_duty_cycle=big_task_duty_cycle)
+
         return {
             f"{cls.task_prefix}_{i}": RTAPhase(
                 prop_wload=PeriodicWload(
@@ -581,7 +709,7 @@ class TwoBigTasks(EASBehaviour):
         }
 
 
-class TwoBigThreeSmall(EASBehaviour):
+class TwoBigThreeSmall(EASBehaviourNoEWMA):
     """
     A mix of 'big' and 'small' tasks
     """
@@ -590,16 +718,15 @@ class TwoBigThreeSmall(EASBehaviour):
     big_prefix = "big"
 
     @classmethod
-    def _get_rtapp_profile(cls, plat_info):
-        little = cls.get_little_cpu(plat_info)
-        big_duty = cls.get_big_duty_cycle(plat_info)
+    def _do_get_rtapp_profile(cls, plat_info, big_task_duty_cycle=None):
+        little_duty = cls.get_little_duty_cycle(plat_info)
+        big_duty = cls.get_big_duty_cycle(plat_info, big_task_duty_cycle=big_task_duty_cycle)
 
         return {
             **{
                 f"{cls.small_prefix}_{i}": RTAPhase(
                     prop_wload=PeriodicWload(
-                        duty_cycle_pct=50,
-                        scale_for_cpu=little,
+                        duty_cycle_pct=little_duty,
                         duration=1,
                         period=cls.TASK_PERIOD
                     )
@@ -619,7 +746,7 @@ class TwoBigThreeSmall(EASBehaviour):
         }
 
 
-class EnergyModelWakeMigration(EASBehaviour):
+class EnergyModelWakeMigration(EASBehaviourNoEWMA):
     """
     One task per big CPU, alternating between two phases:
 
@@ -636,9 +763,9 @@ class EnergyModelWakeMigration(EASBehaviour):
            'Cannot test migration on single capacity group')
 
     @classmethod
-    def _get_rtapp_profile(cls, plat_info):
+    def _do_get_rtapp_profile(cls, plat_info, big_task_duty_cycle=None):
         little = cls.get_little_cpu(plat_info)
-        end_pct = cls.get_big_duty_cycle(plat_info)
+        end_pct = cls.get_big_duty_cycle(plat_info, big_task_duty_cycle=big_task_duty_cycle)
         bigs = plat_info["capacity-classes"][-1]
 
         return {
@@ -663,7 +790,7 @@ class EnergyModelWakeMigration(EASBehaviour):
         }
 
 
-class RampUp(EASBehaviour):
+class RampUp(EASBehaviourNoEWMA):
     """
     A single task whose utilization slowly ramps up
     """
@@ -690,10 +817,10 @@ class RampUp(EASBehaviour):
             capacity_margin_pct=capacity_margin_pct)
 
     @classmethod
-    def _get_rtapp_profile(cls, plat_info):
+    def _do_get_rtapp_profile(cls, plat_info, big_task_duty_cycle=None):
         little = cls.get_little_cpu(plat_info)
         start_pct = cls.unscaled_utilization(plat_info, little, 10)
-        end_pct = cls.get_big_duty_cycle(plat_info)
+        end_pct = cls.get_big_duty_cycle(plat_info, big_task_duty_cycle=big_task_duty_cycle)
 
         return {
             cls.task_name: DutyCycleSweepPhase(
@@ -742,9 +869,9 @@ class RampDown(EASBehaviour):
             capacity_margin_pct=capacity_margin_pct)
 
     @classmethod
-    def _get_rtapp_profile(cls, plat_info):
+    def _get_rtapp_profile(cls, plat_info, big_task_duty_cycle=None):
         little = cls.get_little_cpu(plat_info)
-        start_pct = cls.get_big_duty_cycle(plat_info)
+        start_pct = cls.get_big_duty_cycle(plat_info, big_task_duty_cycle=big_task_duty_cycle)
         end_pct = cls.unscaled_utilization(plat_info, little, 10)
 
         return {
