@@ -18,25 +18,23 @@
 import abc
 import os
 import itertools
+import contextlib
 from statistics import mean
 from typing import TypeVar
 
-import pandas as pd
-import holoviews as hv
+from devlib.exception import TargetStableError
 
 from lisa.tests.base import (
-    TestMetric, Result, ResultBundle, AggregatedResultBundle, TestBundleBase,
-    TestBundle, RTATestBundle
+    Result, ResultBundle, AggregatedResultBundle, TestBundleBase, TestBundle,
+    RTATestBundle
 )
 from lisa.target import Target
-from lisa.utils import ArtifactPath, groupby, ExekallTaggable, add, memoized, kwargs_forwarded_to
-from lisa.datautils import series_mean, df_window, df_filter_task_ids, series_refit_index, df_split_signals, df_refit_index, series_dereference
-from lisa.wlgen.rta import RTA, RTAPhase, PeriodicWload
-from lisa.trace import requires_events, may_use_events, MissingTraceEventError
+from lisa.utils import ArtifactPath, ExekallTaggable, groupby, kwargs_forwarded_to, memoized, ignore_exceps
+from lisa.datautils import df_refit_index, series_dereference, series_mean
+from lisa.wlgen.rta import PeriodicWload, RTAPhase
+from lisa.trace import MissingTraceEventError
 from lisa.analysis.load_tracking import LoadTrackingAnalysis
 from lisa.analysis.tasks import TasksAnalysis
-from lisa.analysis.rta import RTAEventsAnalysis
-from lisa.analysis.frequency import FrequencyAnalysis
 from lisa.pelt import PELT_SCALE, simulate_pelt, pelt_settling_time, kernel_util_mean
 from lisa.notebook import plot_signal
 
@@ -55,46 +53,46 @@ class LoadTrackingHelpers:
 
     MAX_RTAPP_CALIB_DEVIATION = 3 / 100
     """
-    Blacklist CPUs that have a RTapp calibration value that deviates too much
+    Ignore CPUs that have a RTapp calibration value that deviates too much
     from the average calib value in their capacity class.
     """
 
     @classmethod
-    def _get_blacklisted_cpus(cls, plat_info):
+    def _get_ignored_cpus(cls, plat_info):
         """
         :meta public:
 
-        Consider some CPUs as blacklisted when the load would not be
+        Consider some CPUs as ignored when the load would not be
         proportionnal to utilization on them.
 
         That happens for CPUs that are busy executing other code than the test
         workload, like handling interrupts. It is detect that by looking at the
-        RTapp calibration value and we blacklist outliers.
+        RTapp calibration value and we ignore outliers.
         """
         rtapp_calib = plat_info['rtapp']['calib']
-        blacklisted = set()
+        ignored = set()
         # For each class of CPUs, get the average rtapp calibration value
-        # and blacklist the ones that are deviating too much from that
+        # and ignore the ones that are deviating too much from that
         for cpu_class in plat_info['capacity-classes']:
             calib_mean = mean(rtapp_calib[cpu] for cpu in cpu_class)
             calib_max = (1 + cls.MAX_RTAPP_CALIB_DEVIATION) * calib_mean
-            blacklisted.update(
+            ignored.update(
                 cpu
                 for cpu in cpu_class
                 # exclude outliers that are too slow (i.e. calib value too small)
                 if rtapp_calib[cpu] > calib_max
             )
-        return sorted(blacklisted)
+        return sorted(ignored)
 
     @classmethod
     def filter_capacity_classes(cls, plat_info):
         """
-        Filter out capacity-classes key of ``plat_info`` to remove blacklisted
+        Filter out capacity-classes key of ``plat_info`` to remove ignored
         CPUs provided by:
         """
-        blacklisted_cpus = set(cls._get_blacklisted_cpus(plat_info))
+        ignored_cpus = set(cls._get_ignored_cpus(plat_info))
         return [
-            sorted(set(cpu_class) - blacklisted_cpus)
+            sorted(set(cpu_class) - ignored_cpus)
             for cpu_class in plat_info['capacity-classes']
         ]
 
@@ -218,6 +216,9 @@ class InvarianceItemBase(RTATestBundle, LoadTrackingHelpers, TestBundle, Exekall
         # task will fit even at the lowest OPP
         duty_cycle_pct //= 2
 
+        # Catch rt-app calibration induced issues early.
+        assert duty_cycle_pct > 0
+
         return {
             f"{cls.task_prefix}{cpu}": RTAPhase(
                 prop_wload=PeriodicWload(
@@ -289,7 +290,7 @@ class InvarianceItemBase(RTATestBundle, LoadTrackingHelpers, TestBundle, Exekall
             containing the simulated signal, along with the column of the
             signal as found in the trace.
         """
-        logger = self.get_logger()
+        logger = self.logger
         trace = self.trace
         task = trace.get_task_id(task)
 
@@ -314,7 +315,7 @@ class InvarianceItemBase(RTATestBundle, LoadTrackingHelpers, TestBundle, Exekall
         df_activation = df_activation[df.index[0]:]
 
         # Get the initial signal value matching the first activation we will care about
-        init_iloc = df.index.get_loc(df_activation.index[0], method='ffill')
+        init_iloc = df.index.get_indexer([df_activation.index[0]], method='ffill')[0]
         init = df[signal_name].iloc[init_iloc]
 
         try:
@@ -337,6 +338,13 @@ class InvarianceItemBase(RTATestBundle, LoadTrackingHelpers, TestBundle, Exekall
         except MissingTraceEventError:
             capacity = None
         else:
+            capacity = capacity[['cpu', 'capacity_curr']]
+            # We are interested in the current CPU capacity as seen by CFS.
+            # This takes into account:
+            # * The frequency
+            # * The capacity of other sched classes (RT, IRQ etc)
+            capacity = capacity.rename(columns={'capacity_curr': 'capacity'})
+
             # Reshape the capacity dataframe so that we get one column per CPU
             capacity = capacity.pivot(columns=['cpu'])
             capacity.columns = capacity.columns.droplevel(0)
@@ -403,6 +411,7 @@ class InvarianceItemBase(RTATestBundle, LoadTrackingHelpers, TestBundle, Exekall
 
     @memoized
     @get_simulated_pelt.used_events
+    @RTATestBundle.test_noisy_tasks.undecided_filter(noise_threshold_pct=1)
     def _test_correctness(self, signal_name, mean_error_margin_pct, max_error_margin_pct):
 
         task = self.task_name
@@ -513,7 +522,7 @@ class InvarianceBase(TestBundleBase, LoadTrackingHelpers, abc.ABC):
             try:
                 return filtered_class[0]
             except IndexError:
-                raise RuntimeError(f'All CPUs of one capacity class have been blacklisted: {cpu_class}')
+                raise RuntimeError(f'All CPUs of one capacity class have been ignored: {cpu_class}')
 
         # pick one CPU per class of capacity
         cpus = [
@@ -563,20 +572,24 @@ class InvarianceBase(TestBundleBase, LoadTrackingHelpers, abc.ABC):
             )
         ))
 
-        for cpu, (all_freqs, freq_list) in sorted(cpu_freqs.items()):
-            for freq in freq_list:
-                item_dir = ArtifactPath.join(res_dir, f"{InvarianceItemBase.task_prefix}_{cpu}@{freq}")
-                os.makedirs(item_dir)
+        with ignore_exceps(
+            (FileNotFoundError, TargetStableError),
+            target.revertable_write_value('/sys/kernel/debug/workqueue/high_prio_wq', '0')
+        ):
+            for cpu, (all_freqs, freq_list) in sorted(cpu_freqs.items()):
+                for freq in freq_list:
+                    item_dir = ArtifactPath.join(res_dir, f"{InvarianceItemBase.task_prefix}_{cpu}@{freq}")
+                    os.makedirs(item_dir)
 
-                logger.info(f'Running experiment for CPU {cpu}@{freq}')
-                yield cls.ITEM_CLS.from_target(
-                    target,
-                    cpu=cpu,
-                    freq=freq,
-                    freq_list=all_freqs,
-                    res_dir=item_dir,
-                    **kwargs,
-                )
+                    logger.info(f'Running experiment for CPU {cpu}@{freq}')
+                    yield cls.ITEM_CLS.from_target(
+                        target,
+                        cpu=cpu,
+                        freq=freq,
+                        freq_list=all_freqs,
+                        res_dir=item_dir,
+                        **kwargs,
+                    )
 
     def iter_invariance_items(self) -> 'ITEM_CLS':
         yield from self.invariance_items
@@ -660,6 +673,7 @@ class TaskInvariance(InvarianceBase):
 
         @memoized
         @InvarianceItemBase.get_simulated_pelt.used_events
+        @RTATestBundle.test_noisy_tasks.undecided_filter(noise_threshold_pct=1)
         def _test_behaviour(self, signal_name, error_margin_pct):
 
             task = self.task_name
@@ -783,7 +797,7 @@ class TaskInvariance(InvarianceBase):
         .. seealso:: :class:`TaskInvariance.ITEM_CLS.test_util_behaviour`
         """
 
-        logger = self.get_logger()
+        logger = self.logger
 
         def make_group_bundle(cpu, item_group):
             bundle = AggregatedResultBundle(
