@@ -22,6 +22,8 @@ import abc
 import contextlib
 import sqlite3
 import pathlib
+import warnings
+from functools import lru_cache
 
 import pandas as pd
 
@@ -29,7 +31,7 @@ from wa import discover_wa_outputs, Status
 
 from lisa.version import VERSION_TOKEN
 from lisa.stats import Stats
-from lisa.utils import Loggable, memoized, get_subclasses
+from lisa.utils import Loggable, memoized, get_subclasses, LazyMapping
 from lisa._git import find_shortest_symref, get_commit_message
 from lisa.trace import Trace
 
@@ -213,7 +215,7 @@ class WAOutput(StatsProp, Mapping, Loggable):
                 df = collector.df
             except Exception as e: # pylint: disable=broad-except
                 exceps[collector] = e
-                self.get_logger().debug(f'Could not get dataframe of collector {name}: {e}')
+                self.logger.debug(f'Could not get dataframe of collector {name}: {e}')
             else:
                 dfs.append(df)
 
@@ -250,11 +252,72 @@ class WAOutput(StatsProp, Mapping, Loggable):
         """
         parameters = list(inspect.signature(col).parameters.items())
         return any(
-            param.default == param.empty
+            param.default is param.empty and param.kind not in (param.VAR_POSITIONAL, param.VAR_KEYWORD)
             # The first parameter is provided so we can skip it
             for name, param in parameters[1:]
         )
 
+    @property
+    @memoized
+    def jobs(self):
+        """
+        List containing all the jobs present in the output of 'wa run'.
+        """
+        if len(self._jobs) > 1:
+            raise ValueError("jobs is only meant to be used with one output of 'wa run'. Please, use _jobs instead.")
+        return list(self._jobs.values())[0][1]
+
+    @property
+    @memoized
+    def _jobs(self):
+        return {
+            name: (output, [
+                job
+                for job in output.jobs
+                if job.status == Status.OK
+            ])
+            for name, output in self.outputs.items()
+        }
+
+    @property
+    @memoized
+    def outputs(self):
+        """
+        Dict containing a mapping of 'wa run' names to
+        :class:`wa.framework.RunOutput` objects.
+        """
+        wa_outputs = list(discover_wa_outputs(self.path))
+
+        if len(wa_outputs) > 1:
+            warnings.warn('Passing a directory containing multiple outputs of "wa run" to WAOutput is deprecated. Please, create one object per run.', DeprecationWarning)
+
+        wa_outputs = {
+            pathlib.Path(
+                wa_output.basepath
+            ).resolve(): wa_output
+            for wa_output in wa_outputs
+        }
+
+        if len(wa_outputs) > 1:
+            common_prefix = pathlib.Path(
+                os.path.commonpath(wa_outputs.keys())
+            )
+        elif wa_outputs:
+            path, = wa_outputs.keys()
+            common_prefix = pathlib.Path(path).parent
+        else:
+            common_prefix = None
+
+        wa_outputs = {
+            str(
+                name.relative_to(common_prefix)
+                if common_prefix else
+                name
+            ): wa_output
+            for name, wa_output in wa_outputs.items()
+        }
+
+        return wa_outputs
 
 class WACollectorBase(StatsProp, Loggable, abc.ABC):
     """
@@ -273,6 +336,13 @@ class WACollectorBase(StatsProp, Loggable, abc.ABC):
 
     _EXPECTED_WORKLOAD_NAME = None
 
+    _PURE_GET_JOB_DF = True
+    """
+    If ``True``, :meth:`WACollectorBase._get_job_df` will be expected to be
+    pure and its result will be cached. If the result depends in any way from
+    user-provided parameters, this should be ``False``.
+    """
+
     def __init__(self, wa_output, df_postprocess=None):
         self.wa_output = wa_output
         self._df_postprocess = df_postprocess or (lambda x: x)
@@ -304,11 +374,6 @@ class WACollectorBase(StatsProp, Loggable, abc.ABC):
               User can inject arbitrary classifier from WA config, so you might
               have some columns that cannot be relied upon.
         """
-        pass
-
-    @property
-    def logger(self):
-        return self.get_logger()
 
     @property
     @memoized
@@ -321,8 +386,6 @@ class WACollectorBase(StatsProp, Loggable, abc.ABC):
     def _get_df(self):
         self.logger.debug(f"Collecting dataframe for {self.NAME}")
 
-        wa_outputs = list(discover_wa_outputs(self.wa_output.path))
-
         def load_df(job):
             def loader(job):
                 cache_path = os.path.join(
@@ -332,12 +395,18 @@ class WACollectorBase(StatsProp, Loggable, abc.ABC):
 
                 # _get_job_df usually returns fairly large dataframes, so cache
                 # the result for faster reloading
-                try:
-                    df = pd.read_parquet(cache_path)
-                except OSError:
-                    df = self._get_job_df(job)
-                    df.to_parquet(cache_path)
-                return df
+
+                get_df = lambda: self._get_job_df(job)
+
+                if self._PURE_GET_JOB_DF:
+                    try:
+                        df = pd.read_parquet(cache_path)
+                    except OSError:
+                        df = get_df()
+                        df.to_parquet(cache_path)
+                    return df
+                else:
+                    return get_df()
 
             try:
                 df = loader(job)
@@ -352,39 +421,12 @@ class WACollectorBase(StatsProp, Loggable, abc.ABC):
             else:
                 return self._df_postprocess(df)
 
-        wa_outputs = {
-            pathlib.Path(
-                wa_output.basepath
-            ).resolve(): wa_output
-            for wa_output in wa_outputs
-        }
-
-        if len(wa_outputs) > 1:
-            common_prefix = pathlib.Path(
-                os.path.commonpath(wa_outputs.keys())
-            )
-        elif wa_outputs:
-            path, = wa_outputs.keys()
-            common_prefix = pathlib.Path(path).parent
-        else:
-            common_prefix = None
-
-        wa_outputs = {
-            str(
-                name.relative_to(common_prefix)
-                if common_prefix else
-                name
-            ): wa_output
-            for name, wa_output in wa_outputs.items()
-        }
-
         dfs = [
             self._add_output_info(wa_output, name, df)
-            for name, wa_output in wa_outputs.items()
+            for name, (wa_output, jobs) in self.wa_output._jobs.items()
             for df in [
                 load_df(job)
-                for job in wa_output.jobs
-                if job.status == Status.OK
+                for job in jobs
             ]
             if df is not None
         ]
@@ -463,6 +505,7 @@ class WAResultsCollector(WACollectorBase):
     def _get_job_df(cls, job):
         df = pd.DataFrame.from_records(
             {
+                **metric.classifiers,
                 'metric': metric.name,
                 'value': metric.value,
                 'unit': metric.units or '',
@@ -552,6 +595,8 @@ class WAEnergyCollector(WAArtifactCollectorBase):
             **kwargs,
         )
 
+def _stub_trace_to_df(df):
+    raise ValueError('trace_to_df was not specified for the TraceCollector')
 
 class WATraceCollector(WAArtifactCollectorBase):
     """
@@ -588,8 +633,9 @@ class WATraceCollector(WAArtifactCollectorBase):
     """
     NAME = 'trace'
     _ARTIFACT_NAME = 'trace-cmd-bin'
+    _PURE_GET_JOB_DF = False
 
-    def __init__(self, wa_output, trace_to_df, **kwargs):
+    def __init__(self, wa_output, trace_to_df=_stub_trace_to_df, **kwargs):
         self._trace_to_df = trace_to_df
         self._trace_kwargs = kwargs
         super().__init__(wa_output, df_postprocess=None)
@@ -598,6 +644,18 @@ class WATraceCollector(WAArtifactCollectorBase):
         trace = Trace(path, **self._trace_kwargs)
         return self._trace_to_df(trace)
 
+    @property
+    def traces(self):
+        """
+        :class:`lisa.utils.LazyMapping` that maps job names & iteration numbers
+        to their corresponding :class:`lisa.trace.Trace`.
+        """
+        return LazyMapping({
+            f"{job.label}-{job.iteration}": lru_cache()(lambda k: Trace(job.get_artifact_path('trace-cmd-bin'), **self._trace_kwargs))
+            for job in self.wa_output.jobs
+        })
+
+
 class WAJankbenchCollector(WAArtifactCollectorBase):
     """
     WA collector for the jankbench frame timings.
@@ -650,3 +708,89 @@ class WAJankbenchCollector(WAArtifactCollectorBase):
             agg_cols=['iteration', 'frame_id'],
             **kwargs,
         )
+
+
+class WASysfsExtractorCollector(WAArtifactCollectorBase):
+    """
+    WA collector for the syfs-extractor augmentation.
+
+    **Example**::
+
+        def pixel6_energy_meter(df):
+            # Keep only CPU's meters
+            df = df[df.value.str.contains('S4M_VDD_CPUCL0|S3M_VDD_CPUCL1|S2M_VDD_CPUCL2')]
+            df[['variable', 'value']] = df.value.str.split(', ', expand=True)
+
+            def _clean_variable(variable):
+                if 'S4M_VDD_CPUCL0' in variable:
+                    return 'little-energy'
+                if 'S3M_VDD_CPUCL1' in variable:
+                    return 'mid-energy'
+                if 'S2M_VDD_CPUCL2' in variable:
+                    return 'big-energy'
+                return ''
+
+            df['variable'] = df['variable'].apply(_clean_variable)
+            df['value'] = df['value'].astype(int)
+            df['unit'] = "bogo-ujoules"
+
+            # Add a total energy variable
+            df = pd.concat([
+                df,
+                pd.DataFrame(data={
+                    'variable': 'total-energy',
+                    'value': [df['value'].sum()]
+                })
+            ])
+            df.ffill(inplace=True)
+
+            return df
+
+        df = WAOutput('.').get_collector(
+                'sysfs-extractor',
+                path='/sys/bus/iio/devices/iio:device0/energy_value',
+                df_postprocess=pixel6_energy_meter
+        ).df
+    """
+    NAME = 'sysfs-extractor'
+
+    def __init__(self, wa_output, path, type='diff', **kwargs):
+        allowed_types = ['diff', 'before', 'after']
+
+        if type not in allowed_types:
+            ValueError(f'{self.__class__.__qualname__} type must be one of {allowed_types}')
+
+        self._ARTIFACT_NAME = f'{path} [{type}]'
+
+        # TODO: WA's sysfs-extractor augmentation can actually use a path
+        # to collect several files not only one...
+        self._filename = os.path.basename(path)
+
+        super().__init__(wa_output, **kwargs)
+
+    def _get_artifact_df(self, path):
+        # WA given path is actually dirname
+        path = os.path.join(path, self._filename)
+
+        # Expects a sysfs/procfs file e.g
+        # $ cat /sys/bus/iio/devices/iio:device0/energy_value
+        # t=199784
+        # CH0(T=199784)[S10M_VDD_TPU], 965182
+        # CH1(T=199784)[VSYS_PWR_MODEM], 23570587
+        # CH2(T=199784)[VSYS_PWR_RFFE], 2850053
+        # CH3(T=199784)[S2M_VDD_CPUCL2], 88055221
+        # CH4(T=199784)[S3M_VDD_CPUCL1], 38098419
+        # CH5(T=199784)[S4M_VDD_CPUCL0], 98955128
+        # CH6(T=199784)[S5M_VDD_INT], 6657870
+        # CH7(T=199784)[S1M_VDD_MIF], 29268952
+        with open(path, 'r') as f:
+            raw_file = f.readlines()
+
+        raw_file = [line.strip() for line in raw_file]
+
+        df = pd.DataFrame(data={
+                'variable': self._filename,
+                'value': raw_file,
+                'unit': ''
+        })
+        return df
