@@ -30,7 +30,7 @@ import inspect
 
 from devlib.utils.misc import list_to_mask
 
-from lisa.utils import Loggable, ArtifactPath, deprecate, PartialInit, sig_bind
+from lisa.utils import ArtifactPath, Loggable, PartialInit, deprecate, destroyablecontextmanager, ContextManagerExit
 
 
 class _WorkloadRunCMDecorator:
@@ -129,7 +129,7 @@ class _WorkloadRunCM(Loggable):
     def __exit__(self, *exc_info):
         inner_exit = self._bg_cmd.__exit__
         wload = self.wload
-        logger = self.get_logger()
+        logger = self.logger
 
         try:
             suppress = inner_exit(*exc_info)
@@ -280,15 +280,6 @@ class Workload(_WorkloadBase, PartialInit, Loggable):
 
     .. note:: A :class:`Workload` instance can be used as a context manager,
       which ensures :meth:`cleanup` is eventually invoked.
-
-    **Usage example**::
-
-        >>> printer = Printer(target, "test")
-        >>> output = printer.run()
-        INFO    : Printer      : Execution start: echo 42
-        INFO    : Printer      : Execution complete
-        >>> print(output)
-        42\r\n
     """
 
     REQUIRED_TOOLS = []
@@ -353,7 +344,7 @@ class Workload(_WorkloadBase, PartialInit, Loggable):
     def _deployed(self):
         return self._setup_cm is not None
 
-    @contextlib.contextmanager
+    @destroyablecontextmanager
     def _setup(self):
         """
         Context manager function called to setup the target before the
@@ -371,11 +362,11 @@ class Workload(_WorkloadBase, PartialInit, Loggable):
         """
         os.makedirs(self.res_dir, exist_ok=True)
         self.target.execute(f'mkdir -p {quote(self.run_dir)}')
-        self.get_logger().info(f"Created workload's run target directory: {self.run_dir}")
+        self.logger.info(f"Created workload's run target directory: {self.run_dir}")
         self.target.install_tools(self.REQUIRED_TOOLS)
         try:
             yield
-        finally:
+        except ContextManagerExit:
             if self._wipe_run_dir:
                 self.wipe_run_dir()
 
@@ -383,7 +374,7 @@ class Workload(_WorkloadBase, PartialInit, Loggable):
                 shutil.rmtree(self.res_dir)
 
     @property
-    @deprecate('Processed output is returned by run() or by the ".output" attribute of the value returned by the run_background() context manager', deprecated_in='2.0', removed_in='3.0')
+    @deprecate('Processed output is returned by run() or by the ".output" attribute of the value returned by the run_background() context manager', deprecated_in='2.0', removed_in='4.0')
     def output(self):
         return self._output
 
@@ -562,7 +553,7 @@ class Workload(_WorkloadBase, PartialInit, Loggable):
             sessions. For other purposes, use :class:`Workload` instances as a
             context manager.
         """
-        logger = self.get_logger()
+        logger = self.logger
         logger.info(f"Wiping target run directory: {self.run_dir}")
         self.target.remove(self.run_dir)
 
@@ -593,7 +584,7 @@ class Workload(_WorkloadBase, PartialInit, Loggable):
         by :class:`Workload`'s ``__init__``, but they can all be overridden
         manually with keyword arguments if that is really necessary.
         """
-        logger = self.get_logger()
+        logger = self.logger
         target = self.target
 
         settings = {
